@@ -47,7 +47,7 @@
 
 #include <libmugrid/ccoord_operations.hh>
 #include <libmugrid/numpy_tools.hh>
-#include <libmufft/communicator.hh>
+#include <libmugrid/communicator.hh>
 
 #ifdef WITH_FFTWMPI
 #include "libmufft/fftwmpi_engine.hh"
@@ -76,10 +76,6 @@ using muSpectre::Rcoord_t;
 using pybind11::literals::operator""_a;
 namespace py = pybind11;
 
-#ifdef WITH_FFTWMPI
-using muFFT::FFTWMPIEngine;
-#endif
-
 /**
  * the cell factory is only bound for default template parameters
  */
@@ -117,16 +113,18 @@ void add_cell_factory(py::module & mod) {
       "nb_grid_pts"_a, "lengths"_a, "formulation"_a);
 }
 
-#ifdef WITH_FFTWMPI
-void add_fftwmpi_cell_factory(py::module & mod) {
+#if defined(WITH_FFTWMPI) || defined(WITH_PFFT)
+template <class Engine>
+void add_cell_factory_for_engine(py::module & mod,
+                                 const char * name) {
   using DynCcoord_t = muGrid::DynCcoord_t;
   using DynRcoord_t = muGrid::DynRcoord_t;
 
   mod.def(
-      "FFTWMPICellFactory",
+      name,
       [](DynCcoord_t res, DynRcoord_t lens, Formulation form,
          Gradient_t gradient, Communicator comm) {
-        return muSpectre::make_cell<Cell, FFTWMPIEngine>(
+        return muSpectre::make_cell<Cell, Engine>(
             std::move(res), std::move(lens), std::move(form),
             std::move(gradient), std::move(comm));
       },
@@ -134,19 +132,19 @@ void add_fftwmpi_cell_factory(py::module & mod) {
       "communicator"_a);
 
   mod.def(
-      "FFTWMPICellFactory",
+      name,
       [](DynCcoord_t res, DynRcoord_t lens, Formulation form,
          Gradient_t gradient) {
-        return muSpectre::make_cell<Cell, FFTWMPIEngine>(
+        return muSpectre::make_cell<Cell, Engine>(
             std::move(res), std::move(lens), std::move(form),
             std::move(gradient));
       },
       "nb_grid_pts"_a, "lengths"_a, "formulation"_a, "gradient"_a);
 
   mod.def(
-      "FFTWMPICellFactory",
+      name,
       [](DynCcoord_t res, DynRcoord_t lens, Formulation form) {
-        return muSpectre::make_cell<Cell, FFTWMPIEngine>(
+        return muSpectre::make_cell<Cell, Engine>(
             std::move(res), std::move(lens), std::move(form));
       },
       "nb_grid_pts"_a, "lengths"_a, "formulation"_a);
@@ -180,26 +178,26 @@ void add_cell_helper(py::module & mod) {
   using DynRcoord_t = muGrid::DynRcoord_t;
 #endif
   auto NumpyT2Proxy{
-      [](Cell & cell,
-         py::array_t<Real, py::array::f_style> & tensor2)
+      [](Cell & cell, py::array_t<Real, py::array::f_style> & tensor2)
           -> NumpyProxy<Real, py::array::f_style> {
         auto && strain_shape{cell.get_strain_shape()};
         auto & proj{cell.get_projection()};
         return NumpyProxy<Real, py::array::f_style>{
+            proj.get_nb_domain_grid_pts(),
             proj.get_nb_subdomain_grid_pts(),
             proj.get_subdomain_locations(),
             proj.get_nb_quad_pts(),
             {strain_shape[0], strain_shape[1]},
             tensor2};
       }};
-  py::class_<Cell>(mod, "Cell")
-      .def(py::init([](const muSpectre::ProjectionBase & projection) {
-        return Cell{projection.clone()};
-      }))
+  py::class_<Cell, muSpectre::MatrixAdaptable, std::shared_ptr<Cell>> tmp(
+      mod, "Cell");
+  tmp.def(py::init([](const muSpectre::ProjectionBase & projection) {
+       return Cell{projection.clone()};
+     }))
       .def("initialise", &Cell::initialise)
-      .def(
-          "is_initialised", [](Cell & s) { return s.is_initialised(); },
-          py::return_value_policy::reference_internal)
+      .def("is_initialised", [](Cell & s) { return s.is_initialised(); },
+           py::return_value_policy::reference_internal)
       .def(
           "directional_stiffness",
           [&NumpyT2Proxy](
@@ -216,8 +214,7 @@ void add_cell_helper(py::module & mod) {
             }
             auto & delta_stress{
                 dynamic_cast<muGrid::RealField &>(fields.get_field(out_name))};
-            auto delta_strain_array{
-                NumpyT2Proxy(cell, delta_strain)};
+            auto delta_strain_array{NumpyT2Proxy(cell, delta_strain)};
             cell.evaluate_projected_directional_stiffness(
                 delta_strain_array.get_field(), delta_stress);
             return numpy_wrap(delta_stress);
@@ -239,9 +236,7 @@ void add_cell_helper(py::module & mod) {
             }
             auto & strain_field{
                 dynamic_cast<muGrid::RealField &>(fields.get_field(out_name))};
-            strain_field =
-                NumpyT2Proxy(cell, strain)
-                    .get_field();
+            strain_field = NumpyT2Proxy(cell, strain).get_field();
             cell.apply_projection(strain_field);
             return numpy_wrap(strain_field);
           },
@@ -253,12 +248,12 @@ void add_cell_helper(py::module & mod) {
       .def_property_readonly("stress", &Cell::get_stress)
       .def_property_readonly("nb_dof", &Cell::get_nb_dof)
       .def_property_readonly("nb_pixels", &Cell::get_nb_pixels)
+      .def_property_readonly("dim", &Cell::get_spatial_dim)
       .def(
           "evaluate_stress_tangent",
           [&NumpyT2Proxy](Cell & cell,
                           py::array_t<Real, py::array::f_style> & strain) {
-            auto strain_array{
-                NumpyT2Proxy(cell, strain)};
+            auto strain_array{NumpyT2Proxy(cell, strain)};
 
             cell.get_strain() = strain_array.get_field();
             auto && stress_tgt{cell.evaluate_stress_tangent()};
@@ -281,42 +276,44 @@ void add_cell_helper(py::module & mod) {
           "evaluate_stress",
           [&NumpyT2Proxy](Cell & cell,
                           py::array_t<Real, py::array::f_style> & strain) {
-            auto strain_array{
-                NumpyT2Proxy(cell, strain)};
+            auto strain_array{NumpyT2Proxy(cell, strain)};
 
             cell.get_strain() = strain_array.get_field();
             return numpy_wrap(cell.evaluate_stress());
-          },
-          "strain"_a, py::keep_alive<0, 1>())
+           },
+           "strain"_a, py::keep_alive<0, 1>())
       .def_property_readonly("projection", &Cell::get_projection)
+      .def_property_readonly("fft_engine", &Cell::get_fft_engine)
       .def_property_readonly("communicator", &Cell::get_communicator)
       .def_property_readonly(
           "nb_subdomain_grid_pts",
           [](Cell & cell) {
-            return cell.get_projection().get_nb_subdomain_grid_pts();
+            return cell.get_nb_subdomain_grid_pts();
           })
       .def_property_readonly(
           "subdomain_locations",
 
           [](Cell & cell) {
-            return cell.get_projection().get_subdomain_locations();
+            return cell.get_subdomain_locations();
           })
       .def_property_readonly(
           "nb_domain_grid_pts",
           [](Cell & cell) {
-            return cell.get_projection().get_nb_domain_grid_pts();
+            return cell.get_nb_domain_grid_pts();
           })
+      .def_property_readonly(
+          "nb_quad_pts",
+          [](Cell & cell) { return cell.get_nb_quad_pts(); })
       .def_property_readonly(
           "domain_lengths",
           [](Cell & cell) {
-            return cell.get_projection().get_domain_lengths();
+            return cell.get_domain_lengths();
           })
-      .def(
-          "set_uniform_strain",
-          [](Cell & cell, py::EigenDRef<Eigen::ArrayXXd> & strain) -> void {
-            cell.set_uniform_strain(strain);
-          },
-          "strain"_a)
+      .def("set_uniform_strain",
+           [](Cell & cell, py::EigenDRef<Eigen::ArrayXXd> & strain) -> void {
+             cell.set_uniform_strain(strain);
+           },
+           "strain"_a)
       .def("save_history_variables", &Cell::save_history_variables)
       .def("get_globalised_internal_real_field",
            &Cell::globalise_real_internal_field, "unique_name"_a,
@@ -373,6 +370,12 @@ void add_cell_helper(py::module & mod) {
       .def_property_readonly("pixel_indices", &Cell::get_pixel_indices)
       .def_property_readonly("quad_pt_indices", &Cell::get_quad_pt_indices)
       .def_property_readonly("fields", &Cell::get_fields)
+      .def("get_field_collection", &Cell::get_field_collection,
+           py::return_value_policy::reference_internal)
+      .def("get_field_collection_field_names", [](Cell & cell){
+           return cell.get_field_collection().list_fields();
+           })
+      .def_property_readonly("formulation", &Cell::get_formulation)
 
 #ifdef WITH_SPLIT
       .def(
@@ -383,16 +386,18 @@ void add_cell_helper(py::module & mod) {
              std::vector<DynRcoord_t> precipitate_vertices) {
             cell.make_pixels_precipitate_for_laminate_material(
                 precipitate_vertices, mat_lam, mat_precipitate_cell,
-                mat_precipitate, mat_matrix);
+                std::dynamic_pointer_cast<muSpectre::MaterialMechanicsBase>(
+                    mat_precipitate),
+                std::dynamic_pointer_cast<muSpectre::MaterialMechanicsBase>(
+                    mat_matrix));
           },
           "material_laminate"_a, "mat_precipitate_cell"_a,
           "material_precipitate"_a, "material_matrix"_a, "vertices"_a)
-      .def(
-          "complete_material_assignemnt_simple",
-          [](Cell & cell, Mat_t & mat_matrix_cell) {
-            cell.complete_material_assignment_simple(mat_matrix_cell);
-          },
-          "material_matrix_cell"_a)
+      .def("complete_material_assignemnt_simple",
+           [](Cell & cell, Mat_t & mat_matrix_cell) {
+             cell.complete_material_assignment_simple(mat_matrix_cell);
+           },
+           "material_matrix_cell"_a)
 #endif
       ;  // NOLINT
 }
@@ -403,7 +408,8 @@ void add_cell_split_helper(py::module & mod) {
   using CellSplit_t = muSpectre::CellSplit;
   using Cell_t = muSpectre::Cell;
   using Mat_t = muSpectre::MaterialBase;
-  py::class_<CellSplit_t, Cell_t>(mod, "CellSplit")
+  py::class_<CellSplit_t, Cell_t, std::shared_ptr<CellSplit_t>>(mod,
+                                                                "CellSplit")
       .def(
           "make_precipitate",
           [](CellSplit_t & cell, Mat_t & mat,
@@ -426,18 +432,21 @@ void add_cell_split_helper(py::module & mod) {
 void add_cell(py::module & mod) {
   add_cell_factory(mod);
 
-  auto cell{mod.def_submodule("cell")};
-  cell.doc() = "bindings for cells and cell factories";
-
 #ifdef WITH_SPLIT
   add_split_cell_factory_helper(mod);
-  add_cell_helper(cell);
-  add_cell_split_helper(cell);
+  add_cell_helper(mod);
+  add_cell_split_helper(mod);
 #else
-  add_cell_helper(cell);
+  add_cell_helper(mod);
 #endif
 
 #ifdef WITH_FFTWMPI
-  add_fftwmpi_cell_factory(mod);
+  add_cell_factory_for_engine<muFFT::FFTWMPIEngine>(
+      mod, "FFTWMPICellFactory");
+#endif
+
+#ifdef WITH_PFFT
+  add_cell_factory_for_engine<muFFT::PFFTEngine>(
+      mod, "PFFTCellFactory");
 #endif
 }
