@@ -46,7 +46,9 @@
 #include <sstream>
 #include <string>
 
+using muGrid::PhysicsDomain;
 using muGrid::RuntimeError;
+using muSpectre::Dim_t;
 using muSpectre::Index_t;
 using muSpectre::Real;
 using pybind11::literals::operator""_a;
@@ -81,6 +83,8 @@ void add_material_hyper_elasto_plastic2_helper(py::module & mod);
 template <Index_t Dim>
 void add_material_stochastic_plasticity_helper(py::module & mod);
 template <Index_t Dim>
+void add_material_phase_field_fracture_helper(py::module & mod);
+template <Index_t Dim>
 void add_material_visco_elastic_ss_helper(py::module & mod);
 template <Index_t Dim>
 void add_material_visco_elastic_damage_ss1_helper(py::module & mod);
@@ -88,6 +92,17 @@ template <Index_t Dim>
 void add_material_visco_elastic_damage_ss2_helper(py::module & mod);
 template <Index_t Dim>
 void add_material_neo_hookean_elastic_helper(py::module & mod);
+template <Index_t Dim>
+void add_material_dunant_helper(py::module & mod);
+template <Index_t Dim>
+void add_material_dunant_t_helper(py::module & mod);
+template <Index_t Dim>
+void add_material_dunant_tc_helper(py::module & mod);
+template <Index_t Dim>
+void add_material_dunant_max_helper(py::module & mod);
+/* ---------------------------------------------------------------------- */
+template <Dim_t Dim>
+void add_material_linear_diffusion_helper(py::module & mod);
 
 #ifdef WITH_SPLIT
 template <Index_t Dim, muSpectre::Formulation Form>
@@ -119,34 +134,38 @@ class PyMaterialBase : public muSpectre::MaterialBase {
 
   void compute_stresses(
       const muGrid::RealField & F, muGrid::RealField & P,
-      const muSpectre::Formulation & form,
       const muSpectre::SplitCell & is_cell_split,
       const muSpectre::StoreNativeStress & store_native_stress) override {
     PYBIND11_OVERLOAD_PURE(
         void,              // Return type
         Parent,            // Parent class
         compute_stresses,  // Name of function in C++ (must match Python name)
-        F, P, form, is_cell_split, store_native_stress);
+        F, P, is_cell_split, store_native_stress);
+  }
+
+  PhysicsDomain get_physics_domain() const override {
+    PYBIND11_OVERLOAD_PURE(
+        PhysicsDomain,        // Return Type
+        Parent,               // Parent class
+        get_physics_domain);  // Name of function in C++ (must match Python name
   }
 
   void compute_stresses_tangent(
       const muGrid::RealField & F, muGrid::RealField & P, muGrid::RealField & K,
-      const muSpectre::Formulation & form,
       const muSpectre::SplitCell & is_cell_split,
       const muSpectre::StoreNativeStress & store_native_stress) override {
     PYBIND11_OVERLOAD_PURE(
         void,              // Return type
         Parent,            // Parent class
         compute_stresses,  // Name of function in C++ (must match Python name)
-        F, P, K, form, is_cell_split, store_native_stress);
+        F, P, K, is_cell_split, store_native_stress);
   }
 
   using DynMatrix_t = typename Parent::DynMatrix_t;
   using StressTangent_t = std::tuple<DynMatrix_t, DynMatrix_t>;
   std::tuple<DynMatrix_t, DynMatrix_t>
   constitutive_law_dynamic(const Eigen::Ref<const DynMatrix_t> & strain,
-                           const size_t & quad_pt_index,
-                           const muSpectre::Formulation & form) override {
+                           const size_t & quad_pt_index) override {
     PYBIND11_OVERLOAD_PURE(
         // Return type
         StressTangent_t,
@@ -155,7 +174,7 @@ class PyMaterialBase : public muSpectre::MaterialBase {
         // Name of function in C++ (must match Python name)
         constitutive_law_dynamic,
         // arguments
-        strain, quad_pt_index, form);
+        strain, quad_pt_index);
   }
 };
 
@@ -245,6 +264,7 @@ void add_material_base_helper(py::module & mod) {
                     std::shared_ptr<muGrid::LocalFieldCollection>>())
       .def("save_history_variables", &Material::save_history_variables)
       .def("list_fields", &Material::list_fields)
+      .def("list_state_fields", &Material::list_state_fields)
       .def("size", &Material::size)
       .def(
           "add_pixel", [](Material & mat, size_t pix) { mat.add_pixel(pix); },
@@ -266,15 +286,21 @@ void add_material_helper(py::module & mod) {
   add_material_linear_elastic4_helper<Dim>(mod);
   add_material_linear_elastic_damage1_helper<Dim>(mod);
   add_material_linear_elastic_damage2_helper<Dim>(mod);
+  add_material_linear_diffusion_helper<Dim>(mod);
   add_material_hyper_elasto_plastic1_helper<Dim>(mod);
   add_material_hyper_elasto_plastic2_helper<Dim>(mod);
   add_material_linear_elastic_generic1_helper<Dim>(mod);
   add_material_linear_elastic_generic2_helper<Dim>(mod);
   add_material_stochastic_plasticity_helper<Dim>(mod);
+  add_material_phase_field_fracture_helper<Dim>(mod);
   add_material_visco_elastic_ss_helper<Dim>(mod);
   add_material_visco_elastic_damage_ss1_helper<Dim>(mod);
   add_material_visco_elastic_damage_ss2_helper<Dim>(mod);
   add_material_neo_hookean_elastic_helper<Dim>(mod);
+  add_material_dunant_helper<Dim>(mod);
+  add_material_dunant_t_helper<Dim>(mod);
+  add_material_dunant_tc_helper<Dim>(mod);
+  add_material_dunant_max_helper<Dim>(mod);
   add_material_evaluator<Dim>(mod);
 
 #ifdef WITH_SPLIT
