@@ -40,9 +40,11 @@ Program grant you additional permission to convey the resulting work.
 import numpy as np
 import itertools
 
+import muGrid
 import muFFT
 
 from . import Formulation
+from . import solvers
 
 
 def make_grid(lengths, nb_grid_pts):
@@ -104,6 +106,239 @@ def complement_periodically(array, dim):
     return out_arr
 
 
+def get_complemented_positions_worker(
+        quantities, rve, F0, strain, mean_strain, formulation, dim,
+        projection=None, disps=None, periodically_complemented=False):
+    """
+    worker function to do the get_complemented_positions task
+    Arguments:
+    quantities -- string that indicates which quantities should be returned
+        'p': placements (node positions)
+        'g': grid positions (including applied homogeneous strain)
+        '0': grid positions (without applied homogeneous strain)
+        'd': nodal displacements
+        'n': nodal nonaffine displacements (without homogeneous
+             displacement field)
+        The placements are displacement plus grid positions including applied
+        strain.
+    rve        -- Cell object
+    F0         -- F0 describes an affine deformation of the undeformed grid i.e.
+                  * rectangular grid: F0=np.eye(dim) which is the default case
+                                      and corresponds to the undeformed grid.
+                  * hexagonal grid 2D: with dy = sqrt(3)/2*dx
+                                    F0 = np.array([[ 1, 1/sqrt(3)],
+                                                   [ 0,     1    ]])
+    strain -- strain Field
+    mean_strain -- The mean stress of the RVE
+    dim -- dimension of the the RVE for example 2 for 2D
+    projection -- Projection Object obtained either from the Cell object
+                  or a Solver object in an upstream function
+    disp -- displacement field
+    periodically_complemented -- bool, decides whether the output quantities
+            are periodically complemented (True) or not (False). The periodic
+            complementation is not available in parallel and will raise an
+            error if choosen. The default is False.
+
+    Returns:
+    Tuple build according to the first argument of the function.
+
+    """
+    comm = rve.communicator
+    serial = False if comm.size > 1 else True
+
+    if not serial and periodically_complemented:
+        raise RuntimeError("Periodically complemented output quantities "
+                           "are currently not implemented in the parallel "
+                           "version. Either compute serial or use the "
+                           "periodically not complemented quantities.")
+
+    if F0 is None:
+        F0 = np.eye(len(rve.nb_domain_grid_pts))
+
+    cell_coords = \
+        np.mgrid[tuple(slice(s, e) for s, e in
+                       zip(rve.subdomain_locations,
+                           np.array(rve.subdomain_locations)
+                           + np.array(rve.nb_subdomain_grid_pts)))]
+    if serial and periodically_complemented:
+        node_coords = np.mgrid[tuple(slice(None, n + 1)
+                                     for n in rve.nb_domain_grid_pts)]
+    else:
+        node_coords = cell_coords
+
+    if projection is None and not(disps is None):
+        displacements = disps.reshape(cell_coords.shape)
+        if periodically_complemented:
+            displacements = complement_periodically(displacements,
+                                                    rve.spatial_dim)
+    elif disps is None and not (projection is None):
+        # Nodal nonaffine displacements
+        displacements = projection.integrate_nonaffine_displacements(strain)\
+                                  .array(muGrid.IterUnit.Pixel)
+        if periodically_complemented:
+            displacements = complement_periodically(displacements, dim)
+    else:
+        if comm.rank == 0:
+            raise RuntimeError(
+                "One and only one of the projection or the displacements" +
+                " should be determined")
+    # Undeformed node positions in the reference configuration,complemented
+    coords = (np.transpose(node_coords) * rve.domain_lengths /
+              rve.nb_domain_grid_pts).T
+    # correct coords for a transformation of the undeformed grid
+    coords = coords.T.dot(F0.T).T
+
+    if formulation == Formulation.small_strain:
+        # The small strain tensor is lacking the identity, so we need to add it
+        # for the final computation of the positions
+        mean_strain += np.identity(dim)
+
+    retval = []
+    for q in quantities:
+        if q == 'p':
+            retval += [coords.T.dot(mean_strain.T).T + displacements]
+        elif q == 'g':
+            retval += [coords.T.dot(mean_strain.T).T]
+        elif q == '0':
+            retval += [coords]
+        elif q == 'd':
+            retval += [coords.T.dot(mean_strain.T).T + displacements - coords]
+        elif q == 'n':
+            retval += [displacements]
+        else:
+            raise RuntimeError("Unknown quantity '{}'".format(q))
+    if len(retval) == 1:
+        return retval[0]
+    else:
+        return tuple(retval)
+
+
+def get_complemented_positions(quantities, rve, F0=None,
+                               periodically_complemented=False):
+    """Takes an RVE (Cell) object and returns the deformed and undeformed nodal
+    positions, complemented periodically.
+
+    The quantities of interest are specified via string. For example
+        get_complemented_positions('pd', rve)
+    will return a tuple containing first the placements and second the
+    displacements. The supported quantities are listed below.
+
+    Arguments:
+    quantities -- string that indicates which quantities should be returned
+        'p': placements (node positions)
+        'g': grid positions (including applied homogeneous strain)
+        '0': grid positions (without applied homogeneous strain)
+        'd': nodal displacements
+        'n': nodal nonaffine displacements (without homogeneous
+             displacement field)
+        The placements are displacement plus grid positions including applied
+        strain.
+    rve        -- Cell object
+    F0         -- F0 describes an affine deformation of the undeformed grid i.e.
+                  * rectangular grid: F0=np.eye(dim) which is the default case
+                                      and corresponds to the undeformed grid.
+                  * hexagonal grid 2D: with dy = sqrt(3)/2*dx
+                                    F0 = np.array([[ 1, 1/sqrt(3)],
+                                                   [ 0,     1    ]])
+    periodically_complemented -- bool, decides whether the output quantities
+            are periodically complemented (True) or not (False). The periodic
+            complementation is not available in parallel and will raise an
+            error if choosen. The default is False.
+
+    Returns:
+    Tuple build according to the first argument of the function.
+    """
+    strain = rve.strain
+    comm = rve.communicator
+    if comm.size == 1:
+        mean_strain = np.mean(
+            strain.array(), axis=tuple(i for i in range(2, len(strain.shape))))
+    elif comm.size > 1:
+        strain_mean_per_core = np.mean(
+            strain.array(), axis=tuple(i for i in range(2, len(strain.shape))))
+        nb_elements_per_core = \
+            np.product(rve.nb_subdomain_grid_pts) * rve.nb_quad_pts
+        nb_global_elements = \
+            np.product(rve.nb_domain_grid_pts) * rve.nb_quad_pts
+        core_weight = nb_elements_per_core / nb_global_elements
+        mean_strain = comm.sum(strain_mean_per_core * core_weight)
+
+    return get_complemented_positions_worker(
+        quantities, rve, F0, strain, mean_strain, rve.formulation,
+        rve.dim, projection=rve.projection,
+        periodically_complemented=periodically_complemented)
+
+
+def get_complemented_positions_class_solver(quantities, rve, solver, F0=None,
+                                            periodically_complemented=False):
+    """Takes an RVE (Cell) object and returns the deformed and undeformed nodal
+    positions, complemented periodically.
+
+    The quantities of interest are specified via string. For example
+        get_complemented_positions('pd', rve)
+    will return a tuple containing first the placements and second the
+    displacements. The supported quantities are listed below.
+
+    Arguments:
+    quantities -- string that indicates which quantities should be returned
+        'p': placements (node positions)
+        'g': grid positions (including applied homogeneous strain)
+        '0': grid positions (without applied homogeneous strain)
+        'd': nodal displacements
+        The placements are displacement plus grid positions including applied
+        strain.
+    rve        -- Cell object
+    solver     -- Solver object(must be derived from SolverBase)
+    F0         -- F0 describes an affine deformation of the undeformed grid i.e.
+                  * rectangular grid: F0=np.eye(dim) which is the default case
+                                      and corresponds to the undeformed grid.
+                  * hexagonal grid 2D: with dy = sqrt(3)/2*dx
+                                    F0 = np.array([[ 1, 1/sqrt(3)],
+                                                   [ 0,     1    ]])
+    periodically_complemented -- bool, decides whether the output quantities
+            are periodically complemented (True) or not (False). The periodic
+            complementation is not available in parallel and will raise an
+            error if choosen. The default is False.
+
+    Returns:
+    Tuple build according to the first argument of the function.
+    """
+    strain = solver.grad.field.array().reshape(solver.grad.field.shape, order="F")
+    comm = rve.communicator
+    if comm.size == 1:
+        mean_strain = np.mean(
+            strain, axis=tuple(i for i in range(2, len(strain.shape))))
+    elif comm.size > 1:
+        strain_mean_per_core = np.mean(
+            strain, axis=tuple(i for i in range(2, len(strain.shape))))
+        nb_elements_per_core = \
+            np.product(rve.get_nb_subdomain_grid_pts) * rve.nb_quad_pts
+        nb_global_elements = \
+            np.product(rve.get_nb_domain_grid_pts) * rve.nb_quad_pts
+        core_weight = nb_elements_per_core / nb_global_elements
+        mean_strain = comm.sum(strain_mean_per_core * core_weight)
+
+    if solver.solver_type == solvers.SolverType.finite_elements:
+        displacements = solver.disp.field.array()
+        return get_complemented_positions_worker(
+            quantities, rve, F0,
+            solver.grad.field,
+            mean_strain,
+            solver.formulation,
+            rve.spatial_dim,
+            disps=displacements,
+            periodically_complemented=periodically_complemented)
+    else:
+        return get_complemented_positions_worker(
+            quantities, rve, F0,
+            solver.grad.field,
+            mean_strain,
+            solver.formulation,
+            rve.spatial_dim,
+            projection=solver.projection,
+            periodically_complemented=periodically_complemented)
+
+
 def get_integrator(fft, gradient_op, grid_spacing):
     """Returns the discrete Fourier-space integration operator as a function
     of the position grid (used to determine the spatial dimension and number
