@@ -167,7 +167,8 @@ namespace muSpectre {
   // this function handles the evaluation of stress
   // in case the cells have materials in which pixels are partially composed of
   // diffferent materials.
-  const muGrid::RealField & CellSplit::evaluate_stress() {
+  const muGrid::RealField &
+  CellSplit::evaluate_stress(const StoreNativeStress & store_native_stress) {
     if (not this->initialised) {
       this->initialise();
     }
@@ -178,8 +179,8 @@ namespace muSpectre {
     // initialised as zero filled tensors
     this->stress.set_zero();
     for (auto & mat : this->materials) {
-      mat->compute_stresses(this->strain, this->stress,
-                            this->get_formulation());
+      mat->compute_stresses(this->strain, this->stress, SplitCell::simple,
+                            store_native_stress);
     }
     return this->stress;
   }
@@ -189,7 +190,8 @@ namespace muSpectre {
   // in case the cells have materials in which pixels are partially composed of
   // diffferent materials.
   std::tuple<const muGrid::RealField &, const muGrid::RealField &>
-  CellSplit::evaluate_stress_tangent() {
+  CellSplit::evaluate_stress_tangent(
+      const StoreNativeStress & store_native_stress) {
     if (not this->initialised) {
       this->initialise();
     }
@@ -211,7 +213,7 @@ namespace muSpectre {
     // retruned by this function
     for (auto & mat : this->materials) {
       mat->compute_stresses_tangent(strain, this->stress, this->tangent.value(),
-                                    this->get_formulation(), SplitCell::simple);
+                                    SplitCell::simple, store_native_stress);
     }
     return std::tie(this->stress, this->tangent.value());
   }
@@ -238,8 +240,7 @@ namespace muSpectre {
     }
   }
 
-  /* ----------------------------------------------------------------------
-   */
+  /* ---------------------------------------------------------------------- */
   MaterialBase & CellSplit::add_material(Material_ptr mat) {
     if (mat->get_material_dimension() != this->get_spatial_dim()) {
       throw RuntimeError(
@@ -262,7 +263,7 @@ namespace muSpectre {
     } else {
       for (auto && mat : this->materials) {
         if (mat->get_name() != material.get_name()) {
-          if (!mat->get_is_initialised()) {
+          if (not mat->is_initialised()) {
             mat->initialise();
           }
         }
@@ -276,6 +277,7 @@ namespace muSpectre {
         material.add_pixel_split(pixel, 1.0 - pixel_assigned_ratio[iterator]);
       }
     }
+    material.initialise();
   }
 
   /* ----------------------------------------------------------------------*/
