@@ -53,6 +53,9 @@ namespace muSpectre {
             parent_field_collection == nullptr
                 ? std::make_shared<muGrid::LocalFieldCollection>(
                       spatial_dimension,
+                      // giving the local field collection the same name as for
+                      // the material
+                      this->name,
                       // setting the map for nb_sub_pts on the fly here to avoid
                       // having to set the number of quadrature points
                       // conditionally in the constructor function body. This
@@ -90,14 +93,13 @@ namespace muSpectre {
 
   /* ---------------------------------------------------------------------- */
   void MaterialBase::compute_stresses(
-      const muGrid::Field & F, muGrid::Field & P, const Formulation & form,
+      const muGrid::Field & F, muGrid::Field & P,
       const SplitCell & is_cell_split,
       const StoreNativeStress & store_native_stress) {
     const auto t2_dim{muGrid::ipow(this->material_dimension, 2)};
     const auto & real_F{muGrid::RealField::safe_cast(F, t2_dim, QuadPtTag)};
     auto & real_P{muGrid::RealField::safe_cast(P, t2_dim, QuadPtTag)};
-    this->compute_stresses(real_F, real_P, form, is_cell_split,
-                           store_native_stress);
+    this->compute_stresses(real_F, real_P, is_cell_split, store_native_stress);
   }
 
   /* ---------------------------------------------------------------------- */
@@ -123,7 +125,8 @@ namespace muSpectre {
 
   /* ---------------------------------------------------------------------- */
   Real MaterialBase::get_assigned_ratio(const size_t & pixel_id) {
-    auto id{this->internal_fields->get_global_to_local_index_map()[pixel_id]};
+    auto id{
+        this->internal_fields->get_global_to_local_index_map().at(pixel_id)};
     auto && tmp{this->assigned_ratio->get_map()};
     return tmp[id];
   }
@@ -136,21 +139,21 @@ namespace muSpectre {
   /* ---------------------------------------------------------------------- */
   void MaterialBase::compute_stresses_tangent(
       const muGrid::Field & F, muGrid::Field & P, muGrid::Field & K,
-      const Formulation & form, const SplitCell & is_cell_split,
+      const SplitCell & is_cell_split,
       const StoreNativeStress & store_native_stress) {
     const auto t2_dim{muGrid::ipow(this->material_dimension, 2)};
     const auto & real_F{muGrid::RealField::safe_cast(F, t2_dim, QuadPtTag)};
     auto & real_P{muGrid::RealField::safe_cast(P, t2_dim, QuadPtTag)};
     auto & real_K{
         muGrid::RealField::safe_cast(K, muGrid::ipow(t2_dim, 2), QuadPtTag)};
-    this->compute_stresses_tangent(real_F, real_P, real_K, form, is_cell_split,
+    this->compute_stresses_tangent(real_F, real_P, real_K, is_cell_split,
                                    store_native_stress);
   }
 
   /* ---------------------------------------------------------------------- */
   auto MaterialBase::get_pixel_indices() const ->
       typename muGrid::LocalFieldCollection::PixelIndexIterable {
-    return this->internal_fields->get_pixel_indices_fast();
+    return this->internal_fields->get_pixel_indices();
   }
 
   /* ---------------------------------------------------------------------- */
@@ -164,6 +167,11 @@ namespace muSpectre {
     return this->internal_fields->list_fields();
   }
 
+  /* ---------------------------------------------------------------------- */
+  std::vector<std::string> MaterialBase::list_state_fields() const {
+    return this->internal_fields->list_state_field_unique_prefixes();
+  }
+
   /* ---------------------------------------------------------------------- */
   muGrid::LocalFieldCollection & MaterialBase::get_collection() {
     return *this->internal_fields;
@@ -179,33 +187,15 @@ namespace muSpectre {
 
   /* ---------------------------------------------------------------------- */
   void MaterialBase::initialise() {
-    if (!this->is_initialised) {
+    if (not this->is_initialised_flag) {
       this->internal_fields->initialise();
-      this->is_initialised = true;
-    } else {
-      std::stringstream err_str{};
-      err_str << "The material " << this->name
-              << " has been already initialised."
-              << "Therefore, it cannot be initialised again" << std::endl;
-      throw RuntimeError(err_str.str());
+      this->is_initialised_flag = true;
     }
   }
 
   /* ---------------------------------------------------------------------- */
-  void MaterialBase::check_small_strain_capability(
-      const StrainMeasure & expected_strain_measure) {
-    if (not(is_objective(expected_strain_measure))) {
-      std::stringstream err_str{};
-      err_str
-          << "The material expected strain measure is: "
-          << expected_strain_measure
-          << ", while in small strain the required strain measure should be "
-             "objective (in order to be obtainable from infinitesimal strain)."
-          << " Accordingly, this material is not meant to be utilized in "
-             "small strain formulation"
-          << std::endl;
-      throw(muGrid::RuntimeError(err_str.str()));
-    }
+  const bool & MaterialBase::is_initialised() const {
+    return this->is_initialised_flag;
   }
 
   /* ---------------------------------------------------------------------- */
@@ -214,8 +204,13 @@ namespace muSpectre {
   }
 
   /* ---------------------------------------------------------------------- */
-  const bool & MaterialBase::get_is_initialised() {
-    return this->is_initialised;
+  void MaterialBase::set_nb_quad_pts(const Index_t & nb_quad_pts) {
+    this->internal_fields->set_nb_sub_pts(QuadPtTag, nb_quad_pts);
+  }
+
+  /* ---------------------------------------------------------------------- */
+  void MaterialBase::set_nb_nodal_pts(const Index_t & nb_nodal_pts) {
+    this->internal_fields->set_nb_sub_pts(NodalPtTag, nb_nodal_pts);
   }
 
 }  // namespace muSpectre
