@@ -51,12 +51,6 @@ namespace muSpectre {
       : Parent{std::move(engine), lengths, gradient, Formulation::small_strain},
         C_ref_holder{std::make_unique<C_t>(C_ref_)}, C_ref{
                                                          *this->C_ref_holder} {
-    for (auto res : this->fft_engine->get_nb_domain_grid_pts()) {
-      if (res % 2 == 0) {
-        throw ProjectionError(
-            "Only an odd number of gridpoints in each direction is supported");
-      }
-    }
     if (C_ref_.rows() != DimS * DimS) {
       throw ProjectionError("Wrong size C_ref_");
     }
@@ -91,7 +85,7 @@ namespace muSpectre {
     muFFT::FFT_freqs<DimS> fft_freqs(
         Ccoord(this->fft_engine->get_nb_domain_grid_pts()),
         Rcoord(this->domain_lengths));
-    for (auto && tup : akantu::zip(this->fft_engine->get_pixels()
+    for (auto && tup : akantu::zip(this->fft_engine->get_fourier_pixels()
                                        .template get_dimensioned_pixels<DimS>(),
                                    this->Ghat)) {
       const auto & ccoord{std::get<0>(tup)};  // pointer to
@@ -120,9 +114,8 @@ namespace muSpectre {
         for (Dim_t j{0}; j < DimS; ++j) {
           for (Dim_t l{0}; l < DimS; ++l) {
             for (Dim_t m{0}; m < DimS; ++m) {
-              Complex & g = get(G, i, j, l, m);
-
-              g = 0.25 * (N(j, m) * xi(i) * xi(l) + N(j, l) * xi(i) * xi(m) +
+              G(i + DimS * j, l + DimS * m) =
+                  0.25 * (N(j, m) * xi(i) * xi(l) + N(j, l) * xi(i) * xi(m) +
                           N(i, m) * xi(j) * xi(l) + N(i, l) * xi(j) * xi(m));
             }
           }
@@ -131,6 +124,7 @@ namespace muSpectre {
     }
     /* New operator end*/
 
+    // Ghat(ξ=0) ← 0
     if (this->get_subdomain_locations() == Ccoord{}) {
       this->Ghat[0].setZero();
     }
@@ -140,8 +134,8 @@ namespace muSpectre {
   std::unique_ptr<ProjectionBase>
   ProjectionApproxGreenOperator<DimS>::clone() const {
     return std::make_unique<ProjectionApproxGreenOperator>(
-        this->get_fft_engine().clone(), this->get_domain_lengths(),
-        this->C_ref, this->get_gradient());
+        this->get_fft_engine().clone(), this->get_domain_lengths(), this->C_ref,
+        this->get_gradient());
   }
 
   template class ProjectionApproxGreenOperator<oneD>;
