@@ -39,7 +39,7 @@
 #include <boost/mpl/list.hpp>
 
 #include "tests.hh"
-#include "mpi_context.hh"
+#include "tests/libmugrid/mpi_context.hh"
 #include <libmufft/fftw_engine.hh>
 #ifdef WITH_FFTWMPI
 #include <libmufft/fftwmpi_engine.hh>
@@ -48,6 +48,8 @@
 #include <libmufft/pfft_engine.hh>
 #endif
 
+#include "mpi_context.hh"
+
 #include <libmugrid/ccoord_operations.hh>
 #include <libmugrid/field_collection.hh>
 #include <libmugrid/field_map_static.hh>
@@ -139,7 +141,8 @@ namespace muFFT {
     auto & ref{fc.register_real_field("reference", Fix::sdim * Fix::sdim)};
     auto & result{fc.register_real_field("result", Fix::sdim * Fix::sdim)};
 
-    fc.initialise(Fix::engine.get_nb_subdomain_grid_pts(),
+    fc.initialise(Fix::engine.get_nb_domain_grid_pts(),
+                  Fix::engine.get_nb_subdomain_grid_pts(),
                   Fix::engine.get_subdomain_locations());
 
     using map_t = muGrid::MatrixFieldMap<Real, Mapping::Mut, Fix::sdim,
@@ -203,48 +206,6 @@ namespace muFFT {
     }
   }
 
-  /* ---------------------------------------------------------------------- */
-  BOOST_AUTO_TEST_CASE(gather_test) {
-    auto & comm{MPIContext::get_context().comm};
-    auto rank{comm.rank()};
-    const Dim_t nb_cols = rank + 1;
-    Eigen::Matrix<Real, Eigen::Dynamic, Eigen::Dynamic> send_mat(2, nb_cols);
-
-    for (int col{0}; col < nb_cols; ++col) {
-      send_mat.col(col) = send_mat.col(col).Ones(2, 1) * (rank + col);
-    }
-
-    auto res{comm.template gather<Real>(send_mat)};
-    int counter{0};
-    for (int lrank{0}; lrank < comm.size(); ++lrank) {
-      for (int col{0}; col < lrank + 1; ++col) {
-        BOOST_CHECK_EQUAL(res(0, counter), lrank + col);
-        counter++;
-      }
-    }
-  }
-
-  /* ---------------------------------------------------------------------- */
-  BOOST_AUTO_TEST_CASE(sum_mat_test) {
-    auto & comm{MPIContext::get_context().comm};
-    auto nb_cores{comm.size()};
-    Eigen::Matrix<Real, Eigen::Dynamic, Eigen::Dynamic> send_mat(2, 3);
-    send_mat(0, 0) = 1.;
-    send_mat(0, 1) = 2.;
-    send_mat(0, 2) = 3.;
-    send_mat(1, 0) = 4.;
-    send_mat(1, 1) = 5.;
-    send_mat(1, 2) = 6.;
-    auto res{comm.template sum_mat<Real>(send_mat)};
-    const auto nb_cols{send_mat.cols()};
-    const auto nb_rows{send_mat.rows()};
-    for (int row{0}; row < nb_rows; row++) {
-      for (int col{0}; col < nb_cols; col++) {
-        BOOST_CHECK_EQUAL(res(row, col), (row * nb_cols + col + 1) * nb_cores);
-      }
-    }
-  }
-
   BOOST_AUTO_TEST_SUITE_END();
 
 }  // namespace muFFT
