@@ -39,6 +39,83 @@ import numpy as np
 
 from python_test_imports import muFFT
 
+
+class DerivativeCheck1d(unittest.TestCase):
+    def setUp(self):
+        self.nb_pts = [23]
+        np.random.seed(7)
+        self.field = np.empty(self.nb_pts, order='f')
+        self.field[:] = np.random.random(self.nb_pts)
+        self.fft = muFFT.FFT(self.nb_pts)
+        self.nb_dof = 1
+        self.fft.create_plan(self.nb_dof)
+        self.fourier_field = self.fft.register_fourier_space_field(
+            "fft_workspace", self.nb_dof)
+        self.fft.fft(self.field, self.fourier_field)
+
+    def test_upwind_differences(self):
+        diffop = muFFT.Stencils1D.upwind
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            ndiff = self.field[(x+1) % nx] - self.field[x]
+            ndiff = np.squeeze(ndiff)
+            self.assertAlmostEqual(diff_field[x], ndiff)
+
+    def test_central_differences(self):
+        diffop = muFFT.Stencils1D.central
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            ndiff = (self.field[(x+1) % nx] - self.field[(x-1) % nx])/2
+            ndiff = np.squeeze(ndiff)
+            self.assertAlmostEqual(diff_field[x], ndiff)
+
+    def test_central_2nd_differences(self):
+        diffop = muFFT.Stencils1D.central_2nd
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            ndiff = self.field[(x+1) % nx] - 2*self.field[x] \
+                + self.field[(x-1) % nx]
+            ndiff = np.squeeze(ndiff)
+            self.assertAlmostEqual(diff_field[x], ndiff)
+
+    def test_sixth_order_central_differences(self):
+        diffop = muFFT.Stencil1D.central6
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            ndiff = - 1/60*self.field[(x-3) % nx] \
+                    + 3/20*self.field[(x-2) % nx] \
+                    - 3/4*self.field[(x-1) % nx] \
+                    + 3/4*self.field[(x+1) % nx] \
+                    - 3/20*self.field[(x+2) % nx] \
+                    + 1/60*self.field[(x+3) % nx]
+            ndiff = np.squeeze(ndiff)
+            self.assertAlmostEqual(diff_field[x], ndiff)
+
+
 class DerivativeCheck2d(unittest.TestCase):
     def setUp(self):
         self.nb_pts = [23, 27]
@@ -56,14 +133,17 @@ class DerivativeCheck2d(unittest.TestCase):
         dz = muFFT.DiscreteDerivative([0, 0, 0], [[[-1, 1]]])
         self.assertTrue(dz.stencil.flags.owndata)
         self.assertTrue(np.allclose(dz.stencil[0, 0, :], [-1, 1]))
+        self.assertTrue(np.allclose(dz.lbounds, [0, 0, 0]))
 
         dy = dz.rollaxes(-1)
         self.assertTrue(np.allclose(dy.stencil[0, :, 0], [-1, 1]))
         self.assertTrue(np.allclose(dy.stencil, [[[-1], [1]]]))
+        self.assertTrue(np.allclose(dy.lbounds, [0, 0, 0]))
 
         dx = dy.rollaxes(-1)
         self.assertTrue(np.allclose(dx.stencil[:, 0, 0], [-1, 1]))
         self.assertTrue(np.allclose(dx.stencil, [[[-1]], [[1]]]))
+        self.assertTrue(np.allclose(dx.lbounds, [0, 0, 0]))
 
         dx = muFFT.DiscreteDerivative([0, 0], [[-0.5, 0.5],
                                                [-0.5, 0.5]])
@@ -103,12 +183,12 @@ class DerivativeCheck2d(unittest.TestCase):
         ndiff = np.squeeze(ndiff)
         for x in range(nx):
             for y in range(ny):
-                self.assertAlmostEqual(diff_field[x,y], ndiff[x,y])
+                self.assertAlmostEqual(diff_field[x, y], ndiff[x, y])
 
     def test_fourier_derivative_2_corner(self):
-        #shift the fourier derivative into the lower left shift=[-1/6, -1/6]
-        #corner. (Here the grid spacing is 1 in each direction, otherwise one
-        #should consider it to give the real space shift correct.)
+        # shift the fourier derivative into the lower left shift=[-1/6, -1/6]
+        # corner. (Here the grid spacing is 1 in each direction, otherwise one
+        # should consider it to give the real space shift correct.)
         shift = np.array([-1/6, -1/6])
         diffop = muFFT.FourierDerivative(2, 0, shift)
         q = self.fft.fftfreq
@@ -119,20 +199,20 @@ class DerivativeCheck2d(unittest.TestCase):
         self.fft.ifft(d * self.fourier_field, diff_field)
         diff_field *= self.fft.normalisation
         self.fft.ifft(1j*2*np.pi*q[0] *
-                              np.exp(1j*2*np.pi*np.einsum("i,i...->...", shift, q)) *
-                              fourier_field_copy, ndiff)
+                      np.exp(1j*2*np.pi*np.einsum("i,i...->...", shift, q)) *
+                      fourier_field_copy, ndiff)
         ndiff *= self.fft.normalisation
         nx, ny = self.nb_pts
         diff_field = np.squeeze(diff_field)
         ndiff = np.squeeze(ndiff)
         for x in range(nx):
             for y in range(ny):
-                self.assertAlmostEqual(diff_field[x,y], ndiff[x,y])
+                self.assertAlmostEqual(diff_field[x, y], ndiff[x, y])
 
     def test_fourier_derivative_2_full(self):
-        #shift the fourier derivative by one grid point in x- and y-direction
-        #shift=[1, 1]. (Here the grid spacing is 1 in each direction, otherwise
-        #one should consider it to give the real space shift correct.)
+        # shift the fourier derivative by one grid point in x- and y-direction
+        # shift=[1, 1]. (Here the grid spacing is 1 in each direction, other-
+        # wise one should consider it to give the real space shift correct.)
         shift = np.array([1, 1])
         diffop = muFFT.FourierDerivative(2, 0, shift)
         q = self.fft.fftfreq
@@ -149,7 +229,8 @@ class DerivativeCheck2d(unittest.TestCase):
         ndiff = np.squeeze(ndiff)
         for x in range(nx):
             for y in range(ny):
-                self.assertAlmostEqual(diff_field[x,y], ndiff[(x+1)%nx, (y+1)%ny])
+                self.assertAlmostEqual(diff_field[x, y],
+                                       ndiff[(x+1) % nx, (y+1) % ny])
 
     def test_upwind_differences_x(self):
         diffop = muFFT.Stencils2D.upwind_x
@@ -162,7 +243,7 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = self.field[(x+1)%nx, y] - self.field[x, y]
+                ndiff = self.field[(x+1) % nx, y] - self.field[x, y]
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
@@ -177,7 +258,7 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = self.field[x, (y+1)%ny] - self.field[x, y]
+                ndiff = self.field[x, (y+1) % ny] - self.field[x, y]
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
@@ -192,7 +273,8 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = self.field[(x+1)%nx, (y+1)%ny] - self.field[x, (y+1)%ny]
+                ndiff = self.field[(x+1) % nx, (y+1) % ny] \
+                    - self.field[x, (y+1) % ny]
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
@@ -207,13 +289,14 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = self.field[(x+1)%nx, (y+1)%ny] - self.field[(x+1)%nx, y]
+                ndiff = self.field[(x+1) % nx, (y+1) % ny] \
+                    - self.field[(x+1) % nx, y]
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
     def test_upwind_differences_y2(self):
         diffop = muFFT.DiscreteDerivative([0, 0], [[-1, 1],
-                                                   [ 0, 0]])
+                                                   [0,  0]])
         q = self.fft.fftfreq
         d = diffop.fourier(q)
         diff_field = np.zeros_like(self.field, order='f')
@@ -223,12 +306,12 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = self.field[x, (y+1)%ny] - self.field[x, y]
+                ndiff = self.field[x, (y+1) % ny] - self.field[x, y]
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
     def test_shifted_upwind_differences_y(self):
-        diffop = muFFT.DiscreteDerivative([0, 0], [[ 0, 0],
+        diffop = muFFT.DiscreteDerivative([0, 0], [[0,  0],
                                                    [-1, 1]])
         q = self.fft.fftfreq
         d = diffop.fourier(q)
@@ -239,7 +322,8 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = self.field[(x+1)%nx, (y+1)%ny] - self.field[(x+1)%nx, y]
+                ndiff = self.field[(x+1) % nx, (y+1) % ny] \
+                    - self.field[(x+1) % nx, y]
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
@@ -254,8 +338,9 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = (self.field[(x+1)%nx, y] - self.field[x, y] \
-                         + self.field[(x+1)%nx, (y+1)%ny] - self.field[x, (y+1)%ny])/2
+                ndiff = (self.field[(x+1) % nx, y] - self.field[x, y]
+                         + self.field[(x+1) % nx, (y+1) % ny]
+                         - self.field[x, (y+1) % ny])/2
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
@@ -271,8 +356,9 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = (self.field[x, (y+1)%ny] - self.field[x, y] \
-                         + self.field[(x+1)%nx, (y+1)%ny] - self.field[(x+1)%nx, y])/2
+                ndiff = (self.field[x, (y+1) % ny] - self.field[x, y]
+                         + self.field[(x+1) % nx, (y+1) % ny]
+                         - self.field[(x+1) % nx, y])/2
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
@@ -287,7 +373,8 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = (self.field[(x+1)%nx, y] - self.field[(x-1)%nx, y])/2
+                ndiff = (self.field[(x+1) % nx, y]
+                         - self.field[(x-1) % nx, y])/2
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
@@ -302,7 +389,8 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = (self.field[x, (y+1)%ny] - self.field[x, (y-1)%ny])/2
+                ndiff = (self.field[x, (y+1) % ny]
+                         - self.field[x, (y-1) % ny])/2
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
@@ -310,6 +398,7 @@ class DerivativeCheck2d(unittest.TestCase):
         diffop = muFFT.DiscreteDerivative([0, -3],
                                           [[-1/60, 3/20, -3/4, 0,
                                             3/4, -3/20, 1/60]])
+        self.assertTrue(np.allclose(diffop.lbounds, [0, -3]))
         q = self.fft.fftfreq
         d = diffop.fourier(q)
         diff_field = np.zeros_like(self.field, order='f')
@@ -319,15 +408,121 @@ class DerivativeCheck2d(unittest.TestCase):
         diff_field = np.squeeze(diff_field)
         for x in range(nx):
             for y in range(ny):
-                ndiff = -1/60*self.field[x, (y-3)%ny] \
-                    +3/20*self.field[x, (y-2)%ny] \
-                    -3/4*self.field[x, (y-1)%ny] \
-                    +3/4*self.field[x, (y+1)%ny] \
-                    -3/20*self.field[x, (y+2)%ny] \
-                    +1/60*self.field[x, (y+3)%ny]
+                ndiff = -1/60*self.field[x, (y-3) % ny] \
+                    + 3/20*self.field[x, (y-2) % ny] \
+                    - 3/4*self.field[x, (y-1) % ny] \
+                    + 3/4*self.field[x, (y+1) % ny] \
+                    - 3/20*self.field[x, (y+2) % ny] \
+                    + 1/60*self.field[x, (y+3) % ny]
                 ndiff = np.squeeze(ndiff)
                 self.assertAlmostEqual(diff_field[x, y], ndiff)
 
+    def test_central_2nd_differences_x(self):
+        diffop = muFFT.Stencils2D.central_2nd_x
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny = self.nb_pts
+
+    def test_convenience_interface_discrete_deriv(self):
+        assert self.nb_dof == 1
+        diffop = muFFT.Stencils2D.upwind_x
+        field = self.field.reshape(self.nb_pts, order='F')
+        diff_field = diffop.apply(field)
+        print('diff_field[0, 0] (python):', diff_field[0, 0])
+        diff_field = np.squeeze(diff_field)
+        nx, ny = self.nb_pts
+        for x in range(nx):
+            for y in range(ny):
+                ndiff = self.field[(x+1) % nx, y] - self.field[x, y]
+                if x == 0 and y == 0:
+                    print('ndiff:', ndiff)
+                ndiff = np.squeeze(ndiff)
+                self.assertAlmostEqual(diff_field[x, y], ndiff)
+
+    def test_central_2nd_differences_y(self):
+        diffop = muFFT.Stencils2D.central_2nd_y
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            for y in range(ny):
+                ndiff = self.field[x, (y+1) % ny] - 2*self.field[x, y] \
+                    + self.field[x, (y-1) % ny]
+                ndiff = np.squeeze(ndiff)
+                self.assertAlmostEqual(diff_field[x, y], ndiff)
+
+    def test_hexagonal_T1_dx(self):
+        diffop = muFFT.Stencils2D.hexagonal_T1_dx
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            for y in range(ny):
+                ndiff = self.field[(x+1) % nx, y] - self.field[x, y]
+                ndiff = np.squeeze(ndiff)
+                self.assertAlmostEqual(diff_field[x, y], ndiff)
+
+    def test_hexagonal_T1_dy(self):
+        diffop = muFFT.Stencils2D.hexagonal_T1_dy
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            for y in range(ny):
+                ndiff = 1/2*(2*self.field[x, (y+1) % ny] - self.field[x, y]
+                             - self.field[(x+1) % nx, y])
+                ndiff = np.squeeze(ndiff)
+                self.assertAlmostEqual(diff_field[x, y], ndiff)
+
+    def test_hexagonal_T2_dx(self):
+        diffop = muFFT.Stencils2D.hexagonal_T2_dx
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            for y in range(ny):
+                ndiff = self.field[(x+1) % nx, (y+1) % ny] \
+                    - self.field[x, (y+1) % ny]
+                ndiff = np.squeeze(ndiff)
+                self.assertAlmostEqual(diff_field[x, y], ndiff)
+
+    def test_hexagonal_T2_dy(self):
+        diffop = muFFT.Stencils2D.hexagonal_T2_dy
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            for y in range(ny):
+                ndiff = 1/2*(-2*self.field[(x+1) % nx, y]
+                             + self.field[x, (y+1) % ny]
+                             + self.field[(x+1) % nx, (y+1) % ny])
+                ndiff = np.squeeze(ndiff)
+                self.assertAlmostEqual(diff_field[x, y], ndiff)
+
+
 class DerivativeCheck3d(unittest.TestCase):
     def setUp(self):
         self.nb_pts = [23, 23, 17]
@@ -351,7 +546,7 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, y, z] - self.field[x, y, z]
+                    ndiff = self.field[(x+1) % nx, y, z] - self.field[x, y, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -367,7 +562,7 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[x, (y+1)%ny, z] - self.field[x, y, z]
+                    ndiff = self.field[x, (y+1) % ny, z] - self.field[x, y, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -383,7 +578,7 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[x, y, (z+1)%nz] - self.field[x, y, z]
+                    ndiff = self.field[x, y, (z+1) % nz] - self.field[x, y, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -399,7 +594,7 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, y, z] - self.field[x, y, z]
+                    ndiff = self.field[(x+1) % nx, y, z] - self.field[x, y, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -415,7 +610,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, (y+1)%ny, z] - self.field[x, (y+1)%ny, z]
+                    ndiff = self.field[(x+1) % nx, (y+1) % ny, z] \
+                        - self.field[x, (y+1) % ny, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -431,7 +627,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, (y+1)%ny, (z+1)%nz] - self.field[x, (y+1)%ny, (z+1)%nz]
+                    ndiff = self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz] \
+                        - self.field[x, (y+1) % ny, (z+1) % nz]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -447,7 +644,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, y, (z+1)%nz] - self.field[x, y, (z+1)%nz]
+                    ndiff = self.field[(x+1) % nx, y, (z+1) % nz] \
+                        - self.field[x, y, (z+1) % nz]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -463,7 +661,7 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[x, (y+1)%ny, z] - self.field[x, y, z]
+                    ndiff = self.field[x, (y+1) % ny, z] - self.field[x, y, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -479,11 +677,11 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, (y+1)%ny, z] - self.field[(x+1)%nx, y, z]
+                    ndiff = self.field[(x+1) % nx, (y+1) % ny, z] \
+                        - self.field[(x+1) % nx, y, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
-
     def test_d_111_101(self):
         diffop = muFFT.Stencils3D.d_111_101
         q = self.fft.fftfreq
@@ -496,7 +694,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, (y+1)%ny, (z+1)%nz] - self.field[(x+1)%nx, y, (z+1)%nz]
+                    ndiff = self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz] \
+                        - self.field[(x+1) % nx, y, (z+1) % nz]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -512,7 +711,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[x, (y+1)%ny, (z+1)%nz] - self.field[x, y, (z+1)%nz]
+                    ndiff = self.field[x, (y+1) % ny, (z+1) % nz] \
+                        - self.field[x, y, (z+1) % nz]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -528,7 +728,7 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[x, y, (z+1)%nz] - self.field[x, y, z]
+                    ndiff = self.field[x, y, (z+1) % nz] - self.field[x, y, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -544,7 +744,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, y, (z+1)%nz] - self.field[(x+1)%nx, y, z]
+                    ndiff = self.field[(x+1) % nx, y, (z+1) % nz] \
+                        - self.field[(x+1) % nx, y, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -560,7 +761,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[(x+1)%nx, (y+1)%ny, (z+1)%nz] - self.field[(x+1)%nx, (y+1)%ny, z]
+                    ndiff = self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz] \
+                        - self.field[(x+1) % nx, (y+1) % ny, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -576,7 +778,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = self.field[x, (y+1)%ny, (z+1)%nz] - self.field[x, (y+1)%ny, z]
+                    ndiff = self.field[x, (y+1) % ny, (z+1) % nz] \
+                        - self.field[x, (y+1) % ny, z]
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -592,10 +795,13 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = (self.field[(x+1)%nx, y, z] - self.field[x, y, z] \
-                             + self.field[(x+1)%nx, (y+1)%ny, z] - self.field[x, (y+1)%ny, z] \
-                             + self.field[(x+1)%nx, y, (z+1)%nz] - self.field[x, y, (z+1)%nz] \
-                             + self.field[(x+1)%nx, (y+1)%ny, (z+1)%nz] - self.field[x, (y+1)%ny, (z+1)%nz])/4
+                    ndiff = (self.field[(x+1) % nx, y, z] - self.field[x, y, z]
+                             + self.field[(x+1) % nx, (y+1) % ny, z]
+                             - self.field[x, (y+1) % ny, z]
+                             + self.field[(x+1) % nx, y, (z+1) % nz]
+                             - self.field[x, y, (z+1) % nz]
+                             + self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz]
+                             - self.field[x, (y+1) % ny, (z+1) % nz])/4
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -611,10 +817,14 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = (self.field[x, (y+1)%ny, z] - self.field[x, y, z] \
-                             + self.field[(x+1)%nx, (y+1)%ny, z] - self.field[(x+1)%nx, y, z] \
-                             + self.field[x, (y+1)%ny, (z+1)%nz] - self.field[x, y, (z+1)%nz] \
-                             + self.field[(x+1)%nx, (y+1)%ny, (z+1)%nz] - self.field[(x+1)%nx, y, (z+1)%nz])/4
+                    ndiff = (self.field[x, (y+1) % ny, z]
+                             - self.field[x, y, z]
+                             + self.field[(x+1) % nx, (y+1) % ny, z]
+                             - self.field[(x+1) % nx, y, z]
+                             + self.field[x, (y+1) % ny, (z+1) % nz]
+                             - self.field[x, y, (z+1) % nz]
+                             + self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz]
+                             - self.field[(x+1) % nx, y, (z+1) % nz])/4
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -630,10 +840,14 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = (self.field[x, y, (z+1)%nz] - self.field[x, y, z] \
-                             + self.field[(x+1)%nx, y, (z+1)%nz] - self.field[(x+1)%nx, y, z] \
-                             + self.field[x, (y+1)%ny, (z+1)%nz] - self.field[x, (y+1)%ny, z] \
-                             + self.field[(x+1)%nx, (y+1)%ny, (z+1)%nz] - self.field[(x+1)%nx, (y+1)%ny, z])/4
+                    ndiff = (self.field[x, y, (z+1) % nz]
+                             - self.field[x, y, z]
+                             + self.field[(x+1) % nx, y, (z+1) % nz]
+                             - self.field[(x+1) % nx, y, z]
+                             + self.field[x, (y+1) % ny, (z+1) % nz]
+                             - self.field[x, (y+1) % ny, z]
+                             + self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz]
+                             - self.field[(x+1) % nx, (y+1) % ny, z])/4
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -649,7 +863,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = (self.field[(x+1)%nx, y, z] - self.field[(x-1)%nx, y, z])/2
+                    ndiff = (self.field[(x+1) % nx, y, z]
+                             - self.field[(x-1) % nx, y, z])/2
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -665,7 +880,8 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = (self.field[x, (y+1)%ny, z] - self.field[x, (y-1)%ny, z])/2
+                    ndiff = (self.field[x, (y+1) % ny, z]
+                             - self.field[x, (y-1) % ny, z])/2
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
@@ -681,9 +897,83 @@ class DerivativeCheck3d(unittest.TestCase):
         for x in range(nx):
             for y in range(ny):
                 for z in range(nz):
-                    ndiff = (self.field[x, y, (z+1)%nz] - self.field[x, y, (z-1)%nz])/2
+                    ndiff = (self.field[x, y, (z+1) % nz]
+                             - self.field[x, y, (z-1) % nz])/2
                     ndiff = np.squeeze(ndiff)
                     self.assertAlmostEqual(diff_field[x, y, z], ndiff)
 
+    def test_convenience_interface_discrete_deriv(self):
+        assert self.nb_dof == 1
+        diffop = muFFT.Stencils3D.central_z
+        field = self.field.reshape([*self.nb_pts], order='F')
+        diff_field = diffop.apply(field)
+        diff_field = np.squeeze(diff_field)
+        nx, ny, nz = self.nb_pts
+        for x in range(nx):
+            for y in range(ny):
+                for z in range(nz):
+                    ndiff = (self.field[x, y, (z+1) % nz]
+                             - self.field[x, y, (z-1) % nz])/2
+                    ndiff = np.squeeze(ndiff)
+                    self.assertAlmostEqual(diff_field[x, y, z], ndiff)
+
+    def test_five_tetraheda_T0_derivative_x(self):
+        diffop = muFFT.Stencils3D.linear_finite_elements_5[0]
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny, nz = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            for y in range(ny):
+                for z in range(nz):
+                    ndiff = (self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz]
+                             + self.field[(x+1) % nx, y, z]
+                             - self.field[x, (y+1) % ny, z]
+                             - self.field[x, y, (z+1) % nz])/2
+                    ndiff = np.squeeze(ndiff)
+                    self.assertAlmostEqual(diff_field[x, y, z], ndiff)
+
+    def test_five_tetraheda_T0_derivative_y(self):
+        diffop = muFFT.Stencils3D.linear_finite_elements_5[1]
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny, nz = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            for y in range(ny):
+                for z in range(nz):
+                    ndiff = (self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz]
+                             - self.field[(x+1) % nx, y, z]
+                             + self.field[x, (y+1) % ny, z]
+                             - self.field[x, y, (z+1) % nz])/2
+                    ndiff = np.squeeze(ndiff)
+                    self.assertAlmostEqual(diff_field[x, y, z], ndiff)
+
+    def test_five_tetraheda_T0_derivative_z(self):
+        diffop = muFFT.Stencils3D.linear_finite_elements_5[2]
+        q = self.fft.fftfreq
+        d = diffop.fourier(q)
+        diff_field = np.zeros_like(self.field, order='f')
+        self.fft.ifft(d * self.fourier_field, diff_field)
+        diff_field *= self.fft.normalisation
+        nx, ny, nz = self.nb_pts
+        diff_field = np.squeeze(diff_field)
+        for x in range(nx):
+            for y in range(ny):
+                for z in range(nz):
+                    ndiff = (self.field[(x+1) % nx, (y+1) % ny, (z+1) % nz]
+                             - self.field[(x+1) % nx, y, z]
+                             - self.field[x, (y+1) % ny, z]
+                             + self.field[x, y, (z+1) % nz])/2
+                    ndiff = np.squeeze(ndiff)
+                    self.assertAlmostEqual(diff_field[x, y, z], ndiff)
+
+
 if __name__ == "__main__":
     unittest.main()
