@@ -41,6 +41,11 @@ import numpy as np
 
 from python_test_imports import muFFT, muGrid
 
+if muFFT.has_mpi:
+    from mpi4py import MPI
+    communicator = muFFT.Communicator(MPI.COMM_WORLD)
+else:
+    communicator = muFFT.Communicator()
 
 class FFT_Check(unittest.TestCase):
     def setUp(self):
@@ -51,11 +56,7 @@ class FFT_Check(unittest.TestCase):
                       ([6, 4, 5], (2, 3)),
                       ([6, 4, 4], (1,))]
 
-        if muFFT.has_mpi:
-            from mpi4py import MPI
-            self.communicator = muFFT.Communicator(MPI.COMM_WORLD)
-        else:
-            self.communicator = muFFT.Communicator()
+        self.communicator = communicator
 
         self.engines = []
         if muFFT.has_mpi:
@@ -97,7 +98,14 @@ class FFT_Check(unittest.TestCase):
                              comm.size*(comm.size+1)/2 + 3*comm.size,
                              msg='{} engine'.format(engine_str))
 
-    def test_forward_transform(self):
+    # Disable this test for now because it requires a lot of memory. This is
+    # because it initializes the pixel_indices array, which is large.
+    # def test_large_transform(self):
+    #     for engine_str in self.engines:
+    #         muFFT.FFT([65536, 65536], fft=engine_str,
+    #                   communicator=self.communicator)
+
+    def test_forward_transform_numpy_interface(self):
         for engine_str in self.engines:
             for nb_grid_pts, dims in self.grids:
                 s = self.communicator.size
@@ -137,7 +145,7 @@ class FFT_Check(unittest.TestCase):
                 err = np.linalg.norm(out_ref - out_msp)
                 self.assertLess(err, tol, msg='{} engine'.format(engine_str))
 
-    def test_reverse_transform(self):
+    def test_reverse_transform_numpy_interface(self):
         for engine_str in self.engines:
             for nb_grid_pts, dims in self.grids:
                 s = self.communicator.size
@@ -217,7 +225,8 @@ class FFT_Check(unittest.TestCase):
                 out_ref = global_out_ref[(..., *engine.fourier_slices)]
 
                 fc = muGrid.GlobalFieldCollection(len(nb_grid_pts))
-                fc.initialise(tuple(engine.nb_subdomain_grid_pts))
+                fc.initialise(tuple(engine.nb_domain_grid_pts),
+                              tuple(engine.nb_subdomain_grid_pts))
                 in_field = fc.register_real_field('in_field', dims)
                 self.assertFalse(in_field.array().flags.owndata)
                 in_field.array(muGrid.Pixel)[...] = global_in_arr[
@@ -349,7 +358,7 @@ class FFT_Check(unittest.TestCase):
                 engine.fft(in_arr, out_msp)
 
                 # Check that the output array does not have a unit first dimension
-                self.assertEqual(np.squeeze(out_msp).shape, engine.nb_fourier_grid_pts),# \
+                self.assertEqual(tuple(out_msp.shape), engine.nb_fourier_grid_pts),# \
                     #                "{} not equal to {}".format(out_msp.shape,
                 #                                            engine.nb_fourier_grid_pts)
                 # TODO(pastewka): I think this test is out of date. the numpy
@@ -358,6 +367,16 @@ class FFT_Check(unittest.TestCase):
                 # self.assertEqual(out_msp.shape, global_out_ref.shape)
                 # self.assertEqual(len(out_msp.shape), len(global_out_ref.shape))
 
+                # Convenience interface that returns an array
+                out_msp = engine.fft(in_arr)
+                # Check that the output array does not have a unit first dimension
+                self.assertEqual(tuple(out_msp.shape), engine.nb_fourier_grid_pts),# \
+
+                # Convenience interface with flattened array (should not give a
+                # segmentation fault)
+                self.assertRaises(RuntimeError,
+                                  lambda: engine.fft(in_arr.ravel()))
+
     def test_nb_components1_reverse_transform(self):
         """
         asserts that the output is of shape ( , ) and not ( , , 1)
@@ -393,7 +412,12 @@ class FFT_Check(unittest.TestCase):
                     "{} not equal to {}".format(out_msp.shape,
                                                 engine.nb_subdomain_grid_pts)
 
-    @unittest.skipIf(muFFT.has_mpi and muFFT.Communicator().size > 1,
+                # Convenience interface with flattened array (should not give a
+                # segmentation fault)
+                self.assertRaises(RuntimeError,
+                                  lambda: engine.ifft(in_arr.ravel()))
+
+    @unittest.skipIf(communicator.size > 1,
                      'MPI parallel FFTs do not support 1D transforms')
     def test_1d_transform(self):
         nb_grid_pts = [128, ]
@@ -488,6 +512,8 @@ class FFT_Check(unittest.TestCase):
         assert np.allclose(ref.real, tested.real)
         assert np.allclose(ref.imag, tested.imag)
 
+    @unittest.skipIf(communicator.size > 1,
+                     'This test only works on a single MPI process')
     def test_strides(self):
         for engine_str in self.engines:
             try:
@@ -500,12 +526,23 @@ class FFT_Check(unittest.TestCase):
                 continue
 
             if engine_str == 'fftw':
-                assert engine.subdomain_strides == (1, 3, 15) # column-major
-                assert engine.fourier_strides == (1, 2, 10) # column-major
+                assert engine.subdomain_strides == (1, 3, 15),\
+                    '{} - {}'.format(engine_str, engine.subdomain_strides) # column-major
+                assert engine.fourier_strides == (1, 2, 10), \
+                    '{} - {}'.format(engine_str, engine.fourier_strides) # column-major
             elif engine_str == 'fftwmpi':
-                assert engine.subdomain_strides == (1, 4, 20) # padding in first dimension
-                assert engine.fourier_strides == (1, 14, 2) # transposed output
+                assert engine.subdomain_strides == (1, 4, 20), \
+                    '{} - {}'.format(engine_str, engine.subdomain_strides) # padding in first dimension
+                assert engine.fourier_strides == (1, 14, 2), \
+                    '{} - {}'.format(engine_str, engine.fourier_strides) # transposed output
+            elif engine_str == 'pfft':
+                assert engine.subdomain_strides == (1, 4, 20), \
+                    '{} - {}'.format(engine_str, engine.subdomain_strides) # padding in first dimension
+                assert engine.fourier_strides == (7, 14, 1), \
+                    '{} - {}'.format(engine_str, engine.fourier_strides) # transposed output
 
+    @unittest.skipIf(communicator.size > 1,
+                     'This test only works on a single MPI process')
     def test_raises_incompatible_buffer(self):
         """
         asserts that the output is of shape ( , ) and not ( , , 1)
@@ -519,5 +556,257 @@ class FFT_Check(unittest.TestCase):
         with self.assertRaises(RuntimeError):
             engine.fft(in_data, out_data)
 
+    def test_zero_grid_pts(self):
+        nb_grid_pts = [3, 3] # Gives one CPU with zero points on 4 processes
+        axes = (0, 1)
+
+        try:
+            engine = muFFT.FFT(nb_grid_pts,
+                               fft='fftwmpi',
+                               communicator=self.communicator)
+            engine.create_plan(1)
+        except AttributeError:
+            # This FFT engine has not been compiled into the code. Skip
+            # test.
+            return
+
+        # We need to transpose the input to np.fft because muFFT
+        # uses column-major while np.fft uses row-major storage
+        np.random.seed(1)
+        global_in_arr = np.random.random(nb_grid_pts)
+        global_out_ref = np.fft.fftn(global_in_arr.T, axes=axes).T
+        out_ref = global_out_ref[(..., *engine.fourier_slices)]
+        in_arr = global_in_arr[(..., *engine.subdomain_slices)]
+
+        tol = 1e-14 * np.prod(nb_grid_pts)
+
+        # Separately test convenience interface
+        out_msp = np.empty(engine.nb_fourier_grid_pts,
+                           dtype=complex, order='f')
+        engine.fft(in_arr, out_msp)
+        err = np.linalg.norm(out_ref - out_msp)
+        self.assertLess(err, tol)
+
+class FFTCheckSerialOnly(unittest.TestCase):
+    def setUp(self):
+        #               v- grid
+        #                      v-components
+        self.grids = [([8, 4], (2, 3)),
+                      ([6, 4], (1,)),
+                      ([6, 4, 5], (2, 3)),
+                      ([6, 4, 4], (1,))]
+
+        if muFFT.has_mpi:
+            from mpi4py import MPI
+            self.communicator = muFFT.Communicator(MPI.COMM_WORLD)
+        else:
+            self.communicator = muFFT.Communicator()
+
+        self.engines = []
+        if muFFT.has_mpi:
+            self.engines = ['fftwmpi', 'pfft']
+        if self.communicator.size == 1:
+            self.engines += ['fftw']
+
+    @unittest.skipIf(communicator.size > 1,
+                     'fftw only')
+    def test_rffth2c_2d_sin(self):
+
+        engine = muFFT.FFT([5,5], fft="fftw", 
+            allow_temporary_buffer=False,
+            allow_destroy_input=True)
+
+
+        in_data = np.cos(np.pi * 2 * np.arange(5) / 5).reshape(1,-1) * np.ones((5,1)) 
+        out_data = np.zeros((5,5), dtype=float)
+
+        # Allocate buffers and create plan for one degree of freedom
+        real_buffer = engine.register_halfcomplex_field(
+            "real-space", 1)
+        fourier_buffer = engine.register_halfcomplex_field(
+            "fourier-space", 1)
+
+        real_buffer.array()[...] = np.cos(np.pi * 2 * np.arange(5) / 5).reshape(1,-1) * np.ones((5,1)) 
+
+        engine.hcfft(real_buffer, fourier_buffer)
+
+        expected = np.zeros((5,5))
+        expected[0,1] = 1/2 / engine.normalisation
+        np.testing.assert_allclose(fourier_buffer, expected, atol=1e-14)
+
+        real_buffer.array()[...] = np.sin(np.pi * 2 * np.arange(5) / 5).reshape(1,-1) * np.ones((5,1)) 
+
+        engine.hcfft(real_buffer, fourier_buffer)
+
+        expected = np.zeros((5,5))
+        
+        # The halfcomplex array is:     
+        # r0, r1, r2, ..., rn/2, i(n+1)/2-1, ..., i2, i1
+        # with r, i the real and imaginary parts of the first half of the complex spectrum
+        # Euler formula: 
+        # sin = - i * 1/2 * (e^qx - e^-qx ) 
+        expected[0,-1] = - 1/2 / engine.normalisation
+        np.testing.assert_allclose(fourier_buffer, expected, atol=1e-14)
+
+
+    @unittest.skipIf(communicator.size > 1,
+                     'fftw only')
+    def test_rffth2c_2d_roundtrip(self):
+        
+        for nb_grid_pts in [(5,5),(4,4), (4,5),(5,4)]:
+            nx, ny = nb_grid_pts
+
+            engine = muFFT.FFT(nb_grid_pts, fft="fftw", 
+                allow_temporary_buffer=False,
+                allow_destroy_input=True)
+
+            # Allocate buffers and create plan for one degree of freedom
+            real_buffer = engine.register_halfcomplex_field(
+                "real-space", 1)
+            fourier_buffer = engine.register_halfcomplex_field(
+                "fourier-space", 1)
+
+            original = np.random.normal(size=nb_grid_pts)
+            real_buffer.array()[...] = original.copy()
+
+            engine.hcfft(real_buffer, fourier_buffer)
+            engine.ihcfft(fourier_buffer, real_buffer)
+            real_buffer.array()[...] *= engine.normalisation
+            np.testing.assert_allclose(real_buffer, original, atol=1e-14)
+
+    @unittest.skipIf(communicator.size > 1, 'fftw only')
+    def test_rffth2c_2d_convenience_interface(self):
+
+        nb_grid_pts = (5,5)
+        nx, ny = nb_grid_pts
+
+        engine = muFFT.FFT(nb_grid_pts, fft="fftw")
+        engine.create_plan(1)
+
+        original = np.random.normal(size=nb_grid_pts)
+        original_backup = original.copy()
+        result_real = np.empty(nb_grid_pts,
+                               dtype=float, order='f')
+        result_fourier = result_real.copy()
+
+        engine.hcfft(original, result_fourier)
+        engine.ihcfft(result_fourier, result_real)
+
+        result_real *= engine.normalisation
+
+        np.testing.assert_allclose(original_backup, original, atol=1e-14)
+        np.testing.assert_allclose(result_real, original, atol=1e-14)
+
+    @unittest.skipIf(communicator.size > 1, 'fftw only')
+    def test_rffth2c_multiple_dofs(self):
+        for nb_grid_pts, dims in self.grids:
+            s = self.communicator.size
+            nb_grid_pts = s*np.array(nb_grid_pts)
+            try:
+                engine = muFFT.FFT(nb_grid_pts,
+                                   fft="fftw",
+                                   communicator=self.communicator)
+                engine.create_plan(np.prod(dims))
+            except AttributeError:
+                # This FFT engine has not been compiled into the code. Skip
+                # test.
+                continue
+
+            if len(nb_grid_pts) == 2:
+                axes = (0, 1)
+            elif len(nb_grid_pts) == 3:
+                axes = (0, 1, 2)
+            else:
+                raise RuntimeError('Cannot handle {}-dim transforms'
+                                   .format(len(nb_grid_pts)))
+
+            # We need to transpose the input to np.fft because muFFT
+            # uses column-major while np.fft uses row-major storage
+            np.random.seed(1)
+            global_in_arr = np.random.random([*dims, *nb_grid_pts])
+
+            in_arr = global_in_arr[(..., *engine.subdomain_slices)]
+
+            tol = 1e-14 * np.prod(nb_grid_pts)
+
+            # Separately test convenience interface
+            out_msp = global_in_arr.copy()
+            result_real = global_in_arr.copy()
+
+            engine.hcfft(in_arr, out_msp)
+            engine.ihcfft(out_msp, result_real)
+            result_real *= engine.normalisation
+
+            np.testing.assert_allclose(result_real, in_arr, atol=1e-14)
+
+    @unittest.skipIf(communicator.size > 1, 'fftw only')
+    def test_rffth2c_1d_roundtrip(self):
+        
+        for nb_grid_pts in [(5,),(4,)]:
+            nx, = nb_grid_pts
+
+            engine = muFFT.FFT(nb_grid_pts, fft="fftw", 
+                allow_temporary_buffer=False,
+                allow_destroy_input=True)
+
+            # Allocate buffers and create plan for one degree of freedom
+            real_buffer = engine.register_halfcomplex_field(
+                "real-space", 1)
+            fourier_buffer = engine.register_halfcomplex_field(
+                "fourier-space", 1)
+
+            original = np.random.normal(size=nb_grid_pts)
+            real_buffer.array()[...] = original.copy()
+
+            engine.hcfft(real_buffer, fourier_buffer)
+            engine.ihcfft(fourier_buffer, real_buffer)
+            real_buffer.array()[...] *= engine.normalisation
+            np.testing.assert_allclose(real_buffer, original, atol=1e-14)
+
+    @unittest.skipIf(communicator.size > 1, 'fftw only')
+    def test_rffth2c_3d_roundtrip(self):
+    
+        for nb_grid_pts in [(5,4,5),(4,5,4), (4,4,5) ,(5,5,5), (4,4,4)]:
+
+            engine = muFFT.FFT(nb_grid_pts, fft="fftw", 
+                allow_temporary_buffer=False,
+                allow_destroy_input=True)
+
+            # Allocate buffers and create plan for one degree of freedom
+            real_buffer = engine.register_halfcomplex_field(
+                "real-space", 1)
+            fourier_buffer = engine.register_halfcomplex_field(
+                "fourier-space", 1)
+
+            original = np.random.normal(size=nb_grid_pts)
+            real_buffer.array()[...] = original.copy()
+
+            engine.hcfft(real_buffer, fourier_buffer)
+            engine.ihcfft(fourier_buffer, real_buffer)
+            real_buffer.array()[...] *= engine.normalisation
+            np.testing.assert_allclose(real_buffer, original, atol=1e-14)
+
+    @unittest.skipIf(communicator.size > 1,
+                     'This test only works on a single MPI process')
+    def test_r2hc_incompatible_engines_raise(self):
+        for engine in self.engines:
+            if engine != "fftw":
+                try:
+                    engine = muFFT.FFT([3,5], fft=engine, 
+                        allow_temporary_buffer=False,
+                        allow_destroy_input=True)
+                except AttributeError: # One of the engines is not installed, skip it
+                    continue 
+                # Allocate buffers and create plan for one degree of freedom
+                real_buffer = engine.register_halfcomplex_field(
+                    "real-space", 1)
+                fourier_buffer = engine.register_halfcomplex_field(
+                    "fourier-space", 1)
+                with self.assertRaises(RuntimeError) as context:
+                    engine.hcfft(real_buffer, fourier_buffer)
+
+                self.assertTrue("not implemented " in str(context.exception), str(context.exception))
+
+
 if __name__ == '__main__':
     unittest.main()
