@@ -35,11 +35,20 @@ Program grant you additional permission to convey the resulting work.
 """
 
 import unittest
-import numpy as np
 
 from python_test_imports import muGrid
+
 from python_common_tests import *
+
+from python_communicator_tests import *
 from python_field_tests import *
 
+RUN_NETCDF_TESTS = @RUN_NETCDF_TESTS@
+if RUN_NETCDF_TESTS:
+    from python_file_io_test import FileIOTest
+
+if 'FileIONetCDF' in dir(muGrid):
+    from python_file_io_test import FileIOTest
+
 if __name__ == '__main__':
     unittest.main()
