@@ -46,11 +46,11 @@ class FieldCheck(unittest.TestCase):
 
     def test_buffer_size_one_quad_pt(self):
         fc = muGrid.GlobalFieldCollection(len(self.nb_grid_pts))
-        fc.initialise(self.nb_grid_pts)
+        fc.initialise(self.nb_grid_pts, self.nb_grid_pts)
         # Single component
         f = fc.register_real_field("test-field", 1)
         self.assertEqual(f.array(muGrid.Pixel).shape,
-                         (1,) + self.nb_grid_pts)
+                         self.nb_grid_pts)
         # Four components
         f2 = fc.register_real_field("test-field2", 4)
         self.assertEqual(f2.array(muGrid.Pixel).shape,
@@ -66,7 +66,7 @@ class FieldCheck(unittest.TestCase):
 
     def test_buffer_size_four_quad_pt(self):
         fc = muGrid.GlobalFieldCollection(len(self.nb_grid_pts), {'quad': 4})
-        fc.initialise(self.nb_grid_pts)
+        fc.initialise(self.nb_grid_pts, self.nb_grid_pts)
         # Single component
         f = fc.register_real_field("test-field", 1, 'quad')
         self.assertEqual(f.array(muGrid.Pixel).shape,
@@ -90,7 +90,7 @@ class FieldCheck(unittest.TestCase):
 
     def test_buffer_access(self):
         fc = muGrid.GlobalFieldCollection(len(self.nb_grid_pts), {'quad': 4})
-        fc.initialise(self.nb_grid_pts)
+        fc.initialise(self.nb_grid_pts, self.nb_grid_pts)
         # Single component
         f = fc.register_real_field("test-field", 1, 'quad')
         arr1 = np.array(f, copy=False)
@@ -106,14 +106,15 @@ class FieldCheck(unittest.TestCase):
     def test_col_major(self):
         dims = (3, 4)
         fc = muGrid.GlobalFieldCollection(len(self.nb_grid_pts),
-                                          self.nb_grid_pts,
+                                          self.nb_grid_pts, self.nb_grid_pts,
                                           (0,) * len(self.nb_grid_pts),
                                           {})
         f = fc.register_real_field("test-field", dims)
         a = np.array(f)
         self.assertTrue(a.flags.f_contiguous)
-        self.assertEqual(a.shape, tuple(list(dims) + list(self.nb_grid_pts)))
+        self.assertEqual(a.shape, tuple(list(dims) + [1,] + list(self.nb_grid_pts)))
         strides = np.append([1], np.cumprod(dims))
+        strides = np.append(strides, strides[-1])
         strides = np.append(strides,
                             strides[-1] * np.cumprod(self.nb_grid_pts))
         strides = 8 * strides[:-1]
@@ -124,39 +125,30 @@ class FieldCheck(unittest.TestCase):
         # things.
         dims = (3, 4)
         fc = muGrid.GlobalFieldCollection(
-            len(self.nb_grid_pts), self.nb_grid_pts,
+            len(self.nb_grid_pts), self.nb_grid_pts, self.nb_grid_pts,
             [0] * len(self.nb_grid_pts),
             muGrid.StorageOrder.RowMajor, {}, muGrid.StorageOrder.RowMajor)
-        f = fc.register_real_field("test-field", dims,
-                                   storage_order=muGrid.StorageOrder.RowMajor)
+        f = fc.register_real_field("test-field", dims)
         a = np.array(f)
         self.assertTrue(a.flags.c_contiguous)
-        self.assertEqual(a.shape, tuple(list(dims) + list(self.nb_grid_pts)))
+        self.assertEqual(a.shape, tuple(list(dims) + [1,] + list(self.nb_grid_pts)))
         strides = np.append([1], np.cumprod(self.nb_grid_pts[::-1]))
+        strides = np.append(strides, strides[-1])
         strides = np.append(strides, strides[-1] * np.cumprod(dims[::-1]))
         strides = 8 * strides[-2::-1]
         self.assertEqual(a.strides, tuple(strides))
 
-    def test_mixed_storage(self):
-        # We need to specify the storage order twice and they mean different
-        # things.
-        dims = (3, 4)
-        fc = muGrid.GlobalFieldCollection(
-            len(self.nb_grid_pts), self.nb_grid_pts,
-            [0] * len(self.nb_grid_pts),
-            muGrid.StorageOrder.RowMajor, {}, muGrid.StorageOrder.ColMajor)
-        f = fc.register_real_field("test-field", dims,
-                                   storage_order=muGrid.StorageOrder.RowMajor)
-        a = np.array(f)
-        self.assertFalse(a.flags.c_contiguous)
-        self.assertFalse(a.flags.f_contiguous)
-        self.assertEqual(a.shape, tuple(list(dims) + list(self.nb_grid_pts)))
-        strides = np.append([1], np.cumprod(dims[::-1]))[::-1]
-        strides = np.append(strides, strides[0] *
-                            np.cumprod(self.nb_grid_pts[::-1])[-2::-1])
-        strides = 8 * np.roll(strides, -1)
-        self.assertEqual(a.strides, tuple(strides))
+    def test_local_field_collection(self):
+        lfc_name = "ArbitraryLocalFieldCollectionName"
+        lfc_field_name = "MyLocalField"
+        fc = muGrid.LocalFieldCollection(3, lfc_name)
+        fc.add_pixel(1)  # add pixel with global_index = 1
+        fc.initialise()
+        field = fc.register_real_field(lfc_field_name, 1, 'pixel')
+        field_array = np.array(field, copy=False)
+        field_array[:] = 5  # assigne value 5 to pixel 1
 
+        self.assertEqual(fc.get_name(), lfc_name)
 
 if __name__ == "__main__":
     unittest.main()
