@@ -47,6 +47,8 @@ from python_material_linear_elastic_generic2_test import \
 from python_comparison_test_material_linear_elastic1 import MatTest as MatTest1
 from python_material_hyper_elasto_plastic2_test import \
     MaterialHyperElastoPlastic2_Check
+from python_material_phase_field_fracture_test import \
+    MaterialPhaseFieldFracture_Check
 from python_field_tests import FieldCollection_Check
 
 from python_exact_reference_elastic_test import LinearElastic_Check
@@ -58,10 +60,10 @@ from python_eshelby_slow_test import MuSpectre_Eshelby_Slow_Check
 
 from python_material_evaluator_test import MaterialEvaluator_Check
 
-# FIXME: @Richard L. - please fix parallel stochastic plasticity model
-# from python_stochastic_plasticity_search_test import \
-#    StochasticPlasticitySearch_Check
+from python_stochastic_plasticity_search_test import \
+    StochasticPlasticitySearch_Check
 
+from python_sensitivity_analysis_test import SensitivityAnalysis_Check
 
 from python_projection_tests import *
 
@@ -69,7 +71,10 @@ from python_cell_tests import CellCheck
 from python_eigen_strain_test import EigenStrainCheck
 from python_eigen_strain_solver_test import EigenStrainSolverCheck
 from python_solver_test import SolverCheck
+from python_solverclass_test import SolverClassCheck
+from python_write_paraview_output_test import *
 
+from python_linear_fe_test import *
 
 if __name__ == '__main__':
     unittest.main()
