@@ -41,6 +41,7 @@
 #include "materials/material_linear_orthotropic.hh"
 #include "cell/cell.hh"
 #include "projection/projection_finite_strain_fast.hh"
+#include "projection/projection_small_strain.hh"
 
 #include <libmufft/fftw_engine.hh>
 
@@ -62,6 +63,7 @@ namespace muSpectre {
   struct MaterialFixture {
     constexpr static Index_t sdim{DimS};
     constexpr static Index_t mdim{DimM};
+
     constexpr static Formulation form{Form};
 
     using Vec_t = Eigen::Matrix<Real, DimM, 1>;
@@ -132,13 +134,13 @@ namespace muSpectre {
       MaterialFixture<DimS, DimM, c1, c2, Form, RndVec, RndRatio>::sdim;
 
   /* ---------------------------------------------------------------------- */
-
-  template <Dim_t DimS, Dim_t DimM, int c1, int c2, Formulation Form,
+  template <Dim_t DimS, Dim_t DimM, int c1, int c2, Formulation Form_,
             bool RndVec = false, bool RndRatio = false>
   struct MaterialFixture_with_ortho_ref
-      : public MaterialFixture<DimS, DimM, c1, c2, Form, RndVec, RndRatio> {
-    using Parent = MaterialFixture<DimS, DimM, c1, c2, Form, RndVec, RndRatio>;
+      : public MaterialFixture<DimS, DimM, c1, c2, Form_, RndVec, RndRatio> {
+    using Parent = MaterialFixture<DimS, DimM, c1, c2, Form_, RndVec, RndRatio>;
     using MaterialRef_t = MaterialLinearOrthotropic<DimM>;
+    constexpr static Formulation Form{Form_};
 
     // Constructor
     MaterialFixture_with_ortho_ref()
@@ -202,6 +204,12 @@ namespace muSpectre {
     MaterialRef_t material_ref;
   };
 
+  /* ---------------------------------------------------------------------- */
+  template <Dim_t DimS, Dim_t DimM, int c1, int c2, Formulation Form_,
+            bool RndVec, bool RndRatio>
+  constexpr Formulation MaterialFixture_with_ortho_ref<
+      DimS, DimM, c1, c2, Form_, RndVec, RndRatio>::Form;
+
   /* ---------------------------------------------------------------------- */
   using mat_list = boost::mpl::list<
       MaterialFixture<twoD, twoD, 1, 3, Formulation::finite_strain, true, true>,
@@ -278,23 +286,24 @@ namespace muSpectre {
 
     using Mat_t = Eigen::Matrix<Real, Fix::mdim, Fix::mdim>;
     using FC_t = muGrid::GlobalFieldCollection;
-    // using Ccoord = Ccoord_t<Fix::sdim>;
+
     FC_t globalfields{Fix::mdim};
     globalfields.set_nb_sub_pts(QuadPtTag, Fix::NbQuadPts());
-    globalfields.initialise(cube, loc);
-    muGrid::MappedT2Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        F1_f{"Transformation Gradient 1", globalfields, QuadPtTag};
-    muGrid::MappedT2Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        P1_f{"Nominal Stress 1", globalfields, QuadPtTag};
-    muGrid::MappedT4Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        K1_f{"Tangent Moduli 1", globalfields, QuadPtTag};
+    globalfields.initialise(cube, cube, loc);
 
-    muGrid::MappedT2Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        F2_f{"Transformation Gradient 2", globalfields, QuadPtTag};
-    muGrid::MappedT2Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        P2_f{"Nominal Stress 2", globalfields, QuadPtTag};
-    muGrid::MappedT4Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        K2_f{"Tangent Moduli 2", globalfields, QuadPtTag};
+    globalfields.register_real_field("Transformation Gradient 1",
+                                     Fix::mdim * Fix::mdim, QuadPtTag);
+    globalfields.register_real_field("Nominal Stress 1", Fix::mdim * Fix::mdim,
+                                     QuadPtTag);
+    globalfields.register_real_field("Tangent Moduli 1",
+                                     muGrid::ipow(Fix::mdim, 4), QuadPtTag);
+
+    globalfields.register_real_field("Transformation Gradient 2",
+                                     Fix::mdim * Fix::mdim, QuadPtTag);
+    globalfields.register_real_field("Nominal Stress 2", Fix::mdim * Fix::mdim,
+                                     QuadPtTag);
+    globalfields.register_real_field("Tangent Moduli 2",
+                                     muGrid::ipow(Fix::mdim, 4), QuadPtTag);
 
     Mat_t zero{Mat_t::Zero()};
     Mat_t F{1e-6 * Mat_t::Random() + Mat_t::Identity()};
@@ -306,33 +315,55 @@ namespace muSpectre {
 
     Index_t pix0{0};
     Real error{0.0};
-    Real tol{1e-12};
+    Real tol{2e-9};
 
-    F1_f.get_map()[pix0] = strain;
-    F2_f.get_map()[pix0] = strain;
+    using traits = MaterialMuSpectre_traits<typename Fix::Material_t>;
+
+    typename traits::StressMap_t grad_map_1(
+        globalfields.get_field("Transformation Gradient 1"));
+    grad_map_1[0] = strain;
+
+    typename traits::StressMap_t grad_map_2(
+        globalfields.get_field("Transformation Gradient 2"));
+    grad_map_2[0] = strain;
 
     mat.add_pixel(pix0, mat_precipitate, mat_matrix, ratio, normal_vec);
     mat_precipitate->add_pixel(pix0);
     mat.initialise();
     mat_precipitate->initialise();
 
+    BOOST_TEST_CHECKPOINT("intermediate");
+
+    mat.set_formulation(Fix::get_form());
+    mat_precipitate->set_formulation(Fix::get_form());
     mat.compute_stresses_tangent(
         globalfields.get_field("Transformation Gradient 1"),
         globalfields.get_field("Nominal Stress 1"),
-        globalfields.get_field("Tangent Moduli 1"), Fix::get_form());
+        globalfields.get_field("Tangent Moduli 1"));
+
     mat_precipitate->compute_stresses_tangent(
         globalfields.get_field("Transformation Gradient 2"),
         globalfields.get_field("Nominal Stress 2"),
-        globalfields.get_field("Tangent Moduli 2"), Fix::get_form());
+        globalfields.get_field("Tangent Moduli 2"));
 
-    error = (P1_f.get_map()[pix0] - P2_f.get_map()[pix0]).norm();
+    typename traits::StressMap_t Pmap_1(
+        globalfields.get_field("Nominal Stress 1"));
+
+    typename traits::StressMap_t Pmap_2(
+        globalfields.get_field("Nominal Stress 2"));
+
+    error = rel_error(Pmap_1[pix0], Pmap_2[pix0]);
     BOOST_CHECK_LT(error, tol);
 
-    error = (K1_f.get_map()[pix0] - K2_f.get_map()[pix0]).norm();
-    BOOST_CHECK_LT(error, tol);
+    typename traits::TangentMap_t Kmap_1(
+        globalfields.get_field("Tangent Moduli 1"));
 
-    F1_f.get_map()[pix0] = strain;
-    F2_f.get_map()[pix0] = strain;
+    typename traits::TangentMap_t Kmap_2(
+        globalfields.get_field("Tangent Moduli 2"));
+
+    error = rel_error(Kmap_1[pix0], Kmap_2[pix0]);
+
+    BOOST_CHECK_LT(error, tol);
   }
 
   /* ---------------------------------------------------------------------- */
@@ -354,20 +385,21 @@ namespace muSpectre {
 
     FC_t globalfields{Fix::mdim};
     globalfields.set_nb_sub_pts(QuadPtTag, Fix::NbQuadPts());
-    globalfields.initialise(cube, loc);
-    muGrid::MappedT2Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        F1_f{"Transformation Gradient 1", globalfields, QuadPtTag};
-    muGrid::MappedT2Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        P1_f{"Nominal Stress 1", globalfields, QuadPtTag};
-    muGrid::MappedT4Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        K1_f{"Tangent Moduli 1", globalfields, QuadPtTag};
+    globalfields.initialise(cube, cube, loc);
 
-    muGrid::MappedT2Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        F2_f{"Transformation Gradient 2", globalfields, QuadPtTag};
-    muGrid::MappedT2Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        P2_f{"Nominal Stress 2", globalfields, QuadPtTag};
-    muGrid::MappedT4Field<Real, Mapping::Mut, Fix::mdim, IterUnit::SubPt>
-        K2_f{"Tangent Moduli 2", globalfields, QuadPtTag};
+    globalfields.register_real_field("Transformation Gradient 1",
+                                     Fix::mdim * Fix::mdim, QuadPtTag);
+    globalfields.register_real_field("Nominal Stress 1", Fix::mdim * Fix::mdim,
+                                     QuadPtTag);
+    globalfields.register_real_field("Tangent Moduli 1",
+                                     muGrid::ipow(Fix::mdim, 4), QuadPtTag);
+
+    globalfields.register_real_field("Transformation Gradient 2",
+                                     Fix::mdim * Fix::mdim, QuadPtTag);
+    globalfields.register_real_field("Nominal Stress 2", Fix::mdim * Fix::mdim,
+                                     QuadPtTag);
+    globalfields.register_real_field("Tangent Moduli 2",
+                                     muGrid::ipow(Fix::mdim, 4), QuadPtTag);
 
     Mat_t zero{Mat_t::Zero()};
     Mat_t F{1e-6 * Mat_t::Random() + Mat_t::Identity()};
@@ -376,13 +408,10 @@ namespace muSpectre {
     if (Fix::get_form() == Formulation::small_strain) {
       strain = {0.5 * ((F * F.transpose()) - Mat_t::Identity())};
     }
-    // using Ccoord = Ccoord_t<Fix::sdim>;
+
     Index_t pix0{0};
     Real error{0.0};
-    Real tol{1e-12};
-
-    F1_f.get_map()[pix0] = strain;
-    F2_f.get_map()[pix0] = strain;
+    Real tol{1e-6};
 
     mat.add_pixel(pix0, mat_precipitate, mat_matrix, ratio, normal_vec);
     mat_ref.add_pixel(pix0);
@@ -391,48 +420,61 @@ namespace muSpectre {
     mat_ref.initialise();
 
     F = Mat_t::Identity();
-    F1_f.get_map()[pix0] = F;
-    F2_f.get_map()[pix0] = F;
 
+    using traits = MaterialMuSpectre_traits<typename Fix::Material_t>;
+
+    typename traits::StressMap_t grad_map_1(
+        globalfields.get_field("Transformation Gradient 1"));
+    grad_map_1[0] = strain;
+
+    typename traits::StressMap_t grad_map_2(
+        globalfields.get_field("Transformation Gradient 2"));
+    grad_map_2[0] = strain;
+
+    mat.set_formulation(Fix::get_form());
+    mat_ref.set_formulation(Fix::get_form());
     mat.compute_stresses_tangent(
         globalfields.get_field("Transformation Gradient 1"),
         globalfields.get_field("Nominal Stress 1"),
-        globalfields.get_field("Tangent Moduli 1"), Fix::get_form());
+        globalfields.get_field("Tangent Moduli 1"));
     mat_ref.compute_stresses_tangent(
         globalfields.get_field("Transformation Gradient 2"),
         globalfields.get_field("Nominal Stress 2"),
-        globalfields.get_field("Tangent Moduli 2"), Fix::get_form());
+        globalfields.get_field("Tangent Moduli 2"));
 
-    error = rel_error(P1_f.get_map()[pix0], P2_f.get_map()[pix0]);
+    typename traits::StressMap_t Pmap_1(
+        globalfields.get_field("Nominal Stress 1"));
+
+    typename traits::StressMap_t Pmap_2(
+        globalfields.get_field("Nominal Stress 2"));
+
+    error = rel_error(Pmap_1[pix0], Pmap_2[pix0]);
     BOOST_CHECK_LT(error, tol);
 
-    error = rel_error(K1_f.get_map()[pix0], K2_f.get_map()[pix0]);
+    typename traits::TangentMap_t Kmap_1(
+        globalfields.get_field("Tangent Moduli 1"));
+
+    typename traits::TangentMap_t Kmap_2(
+        globalfields.get_field("Tangent Moduli 2"));
+
+    error = rel_error(Kmap_1[pix0], Kmap_2[pix0]);
     BOOST_CHECK_LT(error, tol);
 
+    mat.set_formulation(Fix::get_form());
     mat.compute_stresses(globalfields.get_field("Transformation Gradient 1"),
-                         globalfields.get_field("Nominal Stress 1"),
-                         Fix::get_form());
+                         globalfields.get_field("Nominal Stress 1"));
     mat_ref.compute_stresses(
         globalfields.get_field("Transformation Gradient 2"),
-        globalfields.get_field("Nominal Stress 2"), Fix::get_form());
+        globalfields.get_field("Nominal Stress 2"));
 
-    auto a{P1_f.get_map()[pix0]};
-    auto b{P2_f.get_map()[pix0]};
+    auto a{Pmap_1[pix0]};
+    auto b{Pmap_2[pix0]};
 
     error = rel_error(a, b);
     BOOST_CHECK_LT(error, tol);
 
     if (Fix::get_form() == Formulation::finite_strain) {
-      BOOST_CHECK_THROW(mat.compute_stresses_tangent(
-                            globalfields.get_field("Transformation Gradient 1"),
-                            globalfields.get_field("Nominal Stress 1"),
-                            globalfields.get_field("Tangent Moduli 1"),
-                            Formulation::small_strain),
-                        std::runtime_error);
-      BOOST_CHECK_THROW(mat.compute_stresses(
-                            globalfields.get_field("Transformation Gradient 1"),
-                            globalfields.get_field("Nominal Stress 1"),
-                            Formulation::small_strain),
+      BOOST_CHECK_THROW(mat.set_formulation(Formulation::small_strain),
                         std::runtime_error);
     }
   }
@@ -452,12 +494,13 @@ namespace muSpectre {
     DynRcoord_t lengths{length_cell, length_cell};
 
     auto fft_ptr{std::make_unique<muFFT::FFTWEngine>(nb_grid_pts)};
-    auto proj_ptr{std::make_unique<ProjectionFiniteStrainFast<sdim>>(
-        std::move(fft_ptr), lengths)};
-    Cell sys(std::move(proj_ptr));
+    using Proj_t = std::conditional_t<Fix::Form == Formulation::finite_strain,
+                                      ProjectionFiniteStrainFast<sdim>,
+                                      ProjectionSmallStrain<sdim>>;
+    auto proj_ptr{std::make_unique<Proj_t>(std::move(fft_ptr), lengths)};
+    auto sys{std::make_shared<Cell>(std::move(proj_ptr))};
 
-    auto & mat_lam = MaterialLaminate<sdim, Formulation::small_strain>::make(
-        sys, "laminate");
+    auto & mat_lam = MaterialLaminate<sdim, Fix::Form>::make(sys, "laminate");
     auto & mat_precipitate_cell =
         MaterialLinearElastic1<sdim>::make(sys, "preciptiate", 2.5, 0.4);
     auto & mat_matrix_cell =
@@ -482,11 +525,19 @@ namespace muSpectre {
     precipitate_vertices.push_back({x_center - x_length, y_center + y_length});
     precipitate_vertices.push_back({x_center + x_length, y_center + y_length});
 
-    sys.make_pixels_precipitate_for_laminate_material(
+    sys->make_pixels_precipitate_for_laminate_material(
         precipitate_vertices, mat_lam, mat_precipitate_cell, mat_precipitate,
         mat_matrix);
-    sys.complete_material_assignment_simple(mat_matrix_cell);
-    sys.initialise();
+    sys->complete_material_assignment_simple(mat_matrix_cell);
+
+    std::stringstream form_msg{};
+    form_msg << "System with formulation '" << sys->get_formulation() << "'";
+
+    BOOST_TEST_CHECKPOINT(form_msg.str());
+    std::cout << form_msg.str() << std::endl;
+    sys->initialise();
+    mat_lam.initialise();
+    mat_precipitate_cell.initialise();
 
     BOOST_CHECK_EQUAL(mat_lam.size(),
                       2 * ((top_y - bottom_y) + (right_x - left_x) - 2));
