@@ -33,9 +33,6 @@
  *
  */
 
-#include <boost/mpl/list.hpp>
-#include <Eigen/Dense>
-
 #include "tests.hh"
 #include "libmugrid/test_goodies.hh"
 
@@ -46,6 +43,9 @@
 #include <cell/cell_factory.hh>
 #include <materials/material_linear_elastic1.hh>
 
+#include <boost/mpl/list.hpp>
+#include <Eigen/Dense>
+
 namespace muSpectre {
 
   BOOST_AUTO_TEST_SUITE(cell_base);
@@ -65,13 +65,15 @@ namespace muSpectre {
   };
 
   template <Dim_t DimS, Dim_t DimM, Formulation form>
-  struct CellFixture : Cell {
+  struct CellFixture {
     constexpr static Dim_t sdim{DimS};
     constexpr static Dim_t mdim{DimM};
     constexpr static Formulation formulation{form};
     CellFixture()
-        : Cell{std::move(cell_input(Sizes<DimS>::get_nb_grid_pts(),
-                                    Sizes<DimS>::get_lengths(), form))} {}
+        : cell{std::make_shared<Cell>(
+              std::move(cell_input(Sizes<DimS>::get_nb_grid_pts(),
+                                   Sizes<DimS>::get_lengths(), form)))} {}
+    std::shared_ptr<Cell> cell;
   };
 
   using fixlist =
@@ -89,16 +91,17 @@ namespace muSpectre {
     auto fft_ptr{std::make_unique<muFFT::FFTWEngine>(nb_grid_pts)};
     auto proj_ptr{std::make_unique<ProjectionFiniteStrainFast<dim>>(
         std::move(fft_ptr), lengths)};
-    Cell sys{std::move(proj_ptr)};
+    auto sys{std::make_shared<Cell>(std::move(proj_ptr))};
 
     auto sys2{make_cell(nb_grid_pts, lengths, form)};
     auto sys2b{std::move(sys2)};
-    BOOST_CHECK_EQUAL(sys2b.get_nb_pixels(), sys.get_nb_pixels());
+    BOOST_CHECK_EQUAL(sys2b->get_nb_pixels(), sys->get_nb_pixels());
   }
 
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(constructor_test, fix, fixlist, fix) {
-    BOOST_CHECK_THROW(fix::check_material_coverage(), std::runtime_error);
-    BOOST_CHECK_THROW(fix::initialise(), std::runtime_error);
+    BOOST_CHECK_THROW(this->cell->check_material_coverage(),
+                      std::runtime_error);
+    BOOST_CHECK_THROW(this->cell->initialise(), std::runtime_error);
   }
 
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(add_material_test, fix, fixlist, fix) {
@@ -106,33 +109,7 @@ namespace muSpectre {
     using Material_t = MaterialLinearElastic1<dim>;
     auto Material_hard =
         std::make_unique<Material_t>("hard", dim, OneQuadPt, 210e9, .33);
-    BOOST_CHECK_NO_THROW(fix::add_material(std::move(Material_hard)));
-  }
-
-  BOOST_FIXTURE_TEST_CASE_TEMPLATE(double_initialisation_test, fix, fixlist,
-                                   fix) {
-    constexpr Dim_t dim{fix::sdim};
-    using Material_t = MaterialLinearElastic1<dim>;
-    auto material_hard =
-        std::make_unique<Material_t>("hard", dim, OneQuadPt, 210e9, 0.33);
-    for (const auto & pixel_id : this->get_pixel_indices()) {
-      material_hard->add_pixel(pixel_id);
-    }
-    fix::add_material(std::move(material_hard));
-    fix::initialise();
-    BOOST_CHECK_THROW(fix::initialise(), std::runtime_error);
-  }
-
-  BOOST_FIXTURE_TEST_CASE_TEMPLATE(double_material_initialisation_testt, fix,
-                                   fixlist, fix) {
-    constexpr Dim_t Dim{fix::sdim};
-    using Mat_t = MaterialLinearElastic1<Dim>;
-    auto & material_hard{Mat_t::make(*this, "hard", 210e9, .3)};
-    for (const auto & pixel_id : this->get_pixel_indices()) {
-      material_hard.add_pixel(pixel_id);
-    }
-    material_hard.initialise();
-    BOOST_CHECK_THROW(material_hard.initialise(), std::runtime_error);
+    BOOST_CHECK_NO_THROW(this->cell->add_material(std::move(Material_hard)));
   }
 
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(simple_evaluation_test, fix, fixlist, fix) {
@@ -144,18 +121,18 @@ namespace muSpectre {
     const Real mu{Young / (2 * (1 + Poisson))};
     auto Material_hard =
         std::make_unique<Mat_t>("hard", dim, OneQuadPt, Young, Poisson);
-    for (const auto & pixel_id : this->get_pixel_indices()) {
+    for (const auto & pixel_id : this->cell->get_pixel_indices()) {
       Material_hard->add_pixel(pixel_id);
     }
 
-    fix::add_material(std::move(Material_hard));
+    this->cell->add_material(std::move(Material_hard));
     switch (form) {
     case Formulation::finite_strain: {
-      this->set_uniform_strain(Eigen::Matrix<Real, dim, dim>::Identity());
+      this->cell->set_uniform_strain(Eigen::Matrix<Real, dim, dim>::Identity());
       break;
     }
     case Formulation::small_strain: {
-      this->set_uniform_strain(Eigen::Matrix<Real, dim, dim>::Zero());
+      this->cell->set_uniform_strain(Eigen::Matrix<Real, dim, dim>::Zero());
       break;
     }
     default:
@@ -163,7 +140,7 @@ namespace muSpectre {
       break;
     }
 
-    auto res_tup{fix::evaluate_stress_tangent()};
+    auto res_tup{this->cell->evaluate_stress_tangent()};
     muGrid::T2FieldMap<Real, Mapping::Const, dim, IterUnit::SubPt> stress{
         std::get<0>(res_tup)};
     muGrid::T4FieldMap<Real, Mapping::Const, dim, IterUnit::SubPt> tangent{
@@ -178,7 +155,7 @@ namespace muSpectre {
     }
 
     auto tan_ref = std::get<1>(tup);
-    for (const auto tan : tangent) {
+    for (const auto & tan : tangent) {
       Real norm = (tan - tan_ref).norm();
       BOOST_CHECK_EQUAL(norm, 0.);
     }
@@ -196,21 +173,22 @@ namespace muSpectre {
     const Real mu{Young / (2 * (1 + Poisson))};
     const Real contrast{.5};
 
-    auto & material_hard{Mat_t::make(*this, "hard", Young, Poisson)};
-    auto & material_soft{Mat_t::make(*this, "soft", contrast * Young, Poisson)};
+    auto & material_hard{Mat_t::make(this->cell, "hard", Young, Poisson)};
+    auto & material_soft{
+        Mat_t::make(this->cell, "soft", contrast * Young, Poisson)};
 
-    for (const auto & pixel_id : this->get_pixel_indices()) {
+    for (const auto & pixel_id : this->cell->get_pixel_indices()) {
       static_cast<bool>(pixel_id % 2) ? material_hard.add_pixel(pixel_id)
                                       : material_soft.add_pixel(pixel_id);
     }
 
     switch (form) {
     case Formulation::finite_strain: {
-      this->set_uniform_strain(Eigen::Matrix<Real, dim, dim>::Identity());
+      this->cell->set_uniform_strain(Eigen::Matrix<Real, dim, dim>::Identity());
       break;
     }
     case Formulation::small_strain: {
-      this->set_uniform_strain(Eigen::Matrix<Real, dim, dim>::Zero());
+      this->cell->set_uniform_strain(Eigen::Matrix<Real, dim, dim>::Zero());
       break;
     }
     default:
@@ -218,7 +196,7 @@ namespace muSpectre {
       break;
     }
 
-    auto res_tup{fix::evaluate_stress_tangent()};
+    auto res_tup{this->cell->evaluate_stress_tangent()};
     muGrid::T2FieldMap<Real, Mapping::Const, dim, IterUnit::SubPt> stress{
         std::get<0>(res_tup)};
     muGrid::T4FieldMap<Real, Mapping::Const, dim, IterUnit::SubPt> tangent{
@@ -259,7 +237,7 @@ namespace muSpectre {
     auto Material_soft =
         std::make_unique<Mat_t>("soft", dim, OneQuadPt, 70e9, .3);
 
-    for (const auto & counter : this->get_pixel_indices()) {
+    for (const auto & counter : this->cell->get_pixel_indices()) {
       if (counter < 5) {
         Material_hard->add_pixel(counter);
       } else {
@@ -267,12 +245,12 @@ namespace muSpectre {
       }
     }
 
-    fix::add_material(std::move(Material_hard));
-    fix::add_material(std::move(Material_soft));
+    this->cell->add_material(std::move(Material_hard));
+    this->cell->add_material(std::move(Material_soft));
 
-    fix::evaluate_stress_tangent();
+    this->cell->evaluate_stress_tangent();
 
-    fix::evaluate_stress_tangent();
+    this->cell->evaluate_stress_tangent();
   }
 
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(evaluation_test_new_interface, fix, fixlist,
@@ -284,7 +262,7 @@ namespace muSpectre {
     auto Material_soft =
         std::make_unique<Mat_t>("soft", dim, OneQuadPt, 70e9, .3);
 
-    for (const auto & counter : this->get_pixel_indices()) {
+    for (const auto & counter : this->cell->get_pixel_indices()) {
       if (counter < 5) {
         Material_hard->add_pixel(counter);
       } else {
@@ -292,23 +270,23 @@ namespace muSpectre {
       }
     }
 
-    fix::add_material(std::move(Material_hard));
-    fix::add_material(std::move(Material_soft));
+    this->cell->add_material(std::move(Material_hard));
+    this->cell->add_material(std::move(Material_soft));
 
-    auto F_vec{fix::get_strain().eigen_vec()};
+    auto F_vec{this->cell->get_strain().eigen_vec()};
 
     F_vec.setZero();
 
-    fix::evaluate_stress_tangent();
+    this->cell->evaluate_stress_tangent();
   }
 
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(test_globalised_fields, Fix, fixlist, Fix) {
     constexpr Dim_t Dim{Fix::sdim};
     using Mat_t = MaterialLinearElastic1<Dim>;
-    auto & material_soft{Mat_t::make(*this, "soft", 70e9, .3)};
-    auto & material_hard{Mat_t::make(*this, "hard", 210e9, .3)};
+    auto & material_soft{Mat_t::make(this->cell, "soft", 70e9, .3)};
+    auto & material_hard{Mat_t::make(this->cell, "hard", 210e9, .3)};
 
-    for (const auto & pixel_id : this->get_pixel_indices()) {
+    for (const auto & pixel_id : this->cell->get_pixel_indices()) {
       if (pixel_id % 2) {
         material_soft.add_pixel(pixel_id);
       } else {
@@ -340,11 +318,11 @@ namespace muSpectre {
     pixler(compatible_hard);
 
     auto & global_compatible_field{
-        this->globalise_real_internal_field(compatible_name)};
+        this->cell->globalise_real_internal_field(compatible_name)};
 
     // make sure we get the same field again
     auto & global_compatible_field_again{
-        this->globalise_real_internal_field(compatible_name)};
+        this->cell->globalise_real_internal_field(compatible_name)};
     BOOST_CHECK_EQUAL(&global_compatible_field, &global_compatible_field_again);
 
     auto glo_map{global_compatible_field.get_sub_pt_map()};
@@ -360,14 +338,14 @@ namespace muSpectre {
     const std::string incompatible_name{"incompatible"};
     col_soft.register_real_field(incompatible_name, Dim, QuadPtTag);
 
-    col_hard.register_real_field(incompatible_name, Dim + 1,
-                                 QuadPtTag);
-    BOOST_CHECK_THROW(this->globalise_real_internal_field(incompatible_name),
-                      std::runtime_error);
+    col_hard.register_real_field(incompatible_name, Dim + 1, QuadPtTag);
+    BOOST_CHECK_THROW(
+        this->cell->globalise_real_internal_field(incompatible_name),
+        std::runtime_error);
 
     // wrong name/ inexistant field
     const std::string wrong_name{"wrong_name"};
-    BOOST_CHECK_THROW(this->globalise_real_internal_field(wrong_name),
+    BOOST_CHECK_THROW(this->cell->globalise_real_internal_field(wrong_name),
                       std::runtime_error);
 
     // wrong scalar type:
@@ -375,8 +353,9 @@ namespace muSpectre {
     col_soft.register_real_field(wrong_scalar_name, Dim, QuadPtTag);
 
     col_hard.register_int_field(wrong_scalar_name, Dim, QuadPtTag);
-    BOOST_CHECK_THROW(this->globalise_real_internal_field(wrong_scalar_name),
-                      std::runtime_error);
+    BOOST_CHECK_THROW(
+        this->cell->globalise_real_internal_field(wrong_scalar_name),
+        std::runtime_error);
   }
 
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(test_globalised_current_state_fields, Fix,
@@ -385,10 +364,10 @@ namespace muSpectre {
     const size_t nb_steps_to_save{1};
     const size_t nb_dof{1};
     using Mat_t = MaterialLinearElastic1<Dim>;
-    auto & material_soft{Mat_t::make(*this, "soft", 70e9, .3)};
-    auto & material_hard{Mat_t::make(*this, "hard", 210e9, .3)};
+    auto & material_soft{Mat_t::make(this->cell, "soft", 70e9, .3)};
+    auto & material_hard{Mat_t::make(this->cell, "hard", 210e9, .3)};
 
-    for (const auto & pixel_id : this->get_pixel_indices()) {
+    for (const auto & pixel_id : this->cell->get_pixel_indices()) {
       if (pixel_id % 2) {
         material_soft.add_pixel(pixel_id);
       } else {
@@ -423,11 +402,11 @@ namespace muSpectre {
     pixler(compatible_hard);
 
     auto & global_compatible_field_current{
-        this->globalise_real_current_field(compatible_name)};
+        this->cell->globalise_real_current_field(compatible_name)};
 
     // make sure we get the same field again
     auto & global_compatible_field_again{
-        this->globalise_real_current_field(compatible_name)};
+        this->cell->globalise_real_current_field(compatible_name)};
     BOOST_CHECK_EQUAL(&global_compatible_field_current,
                       &global_compatible_field_again);
 
@@ -448,12 +427,13 @@ namespace muSpectre {
 
     col_hard.register_real_state_field(incompatible_name, nb_steps_to_save,
                                        Dim + 1, QuadPtTag);
-    BOOST_CHECK_THROW(this->globalise_real_current_field(incompatible_name),
-                      std::runtime_error);
+    BOOST_CHECK_THROW(
+        this->cell->globalise_real_current_field(incompatible_name),
+        std::runtime_error);
 
     // wrong name/ inexistant field
     const std::string wrong_name{"wrong_name"};
-    BOOST_CHECK_THROW(this->globalise_real_current_field(wrong_name),
+    BOOST_CHECK_THROW(this->cell->globalise_real_current_field(wrong_name),
                       std::runtime_error);
 
     // wrong scalar type:
@@ -463,8 +443,9 @@ namespace muSpectre {
 
     col_hard.register_int_state_field(wrong_scalar_name, nb_steps_to_save, Dim,
                                       QuadPtTag);
-    BOOST_CHECK_THROW(this->globalise_real_current_field(wrong_scalar_name),
-                      std::runtime_error);
+    BOOST_CHECK_THROW(
+        this->cell->globalise_real_current_field(wrong_scalar_name),
+        std::runtime_error);
   }
 
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(test_globalised_old_state_fields, Fix,
@@ -473,10 +454,10 @@ namespace muSpectre {
     const size_t nb_steps_to_save{1};
     const size_t nb_dof{1};
     using Mat_t = MaterialLinearElastic1<Dim>;
-    auto & material_soft{Mat_t::make(*this, "soft", 70e9, .3)};
-    auto & material_hard{Mat_t::make(*this, "hard", 210e9, .3)};
+    auto & material_soft{Mat_t::make(this->cell, "soft", 70e9, .3)};
+    auto & material_hard{Mat_t::make(this->cell, "hard", 210e9, .3)};
 
-    for (const auto & pixel_id : this->get_pixel_indices()) {
+    for (const auto & pixel_id : this->cell->get_pixel_indices()) {
       if (pixel_id % 2) {
         material_soft.add_pixel(pixel_id);
       } else {
@@ -513,12 +494,12 @@ namespace muSpectre {
     compatible_hard.cycle();
     compatible_soft.cycle();
 
-    auto & global_compatible_field_old{
-        this->globalise_real_old_field(compatible_name, nb_steps_to_save)};
+    auto & global_compatible_field_old{this->cell->globalise_real_old_field(
+        compatible_name, nb_steps_to_save)};
 
     // make sure we get the same field again
-    auto & global_compatible_field_again{
-        this->globalise_real_old_field(compatible_name, nb_steps_to_save)};
+    auto & global_compatible_field_again{this->cell->globalise_real_old_field(
+        compatible_name, nb_steps_to_save)};
     BOOST_CHECK_EQUAL(&global_compatible_field_old,
                       &global_compatible_field_again);
 
@@ -539,12 +520,13 @@ namespace muSpectre {
     col_hard.register_real_state_field(incompatible_name, nb_steps_to_save,
                                        Dim + 1, QuadPtTag);
 
-    BOOST_CHECK_THROW(this->globalise_real_old_field(incompatible_name, 1),
-                      std::runtime_error);
+    BOOST_CHECK_THROW(
+        this->cell->globalise_real_old_field(incompatible_name, 1),
+        std::runtime_error);
 
     // wrong name/ inexistant field
     const std::string wrong_name{"wrong_name"};
-    BOOST_CHECK_THROW(this->globalise_real_old_field(wrong_name, 1),
+    BOOST_CHECK_THROW(this->cell->globalise_real_old_field(wrong_name, 1),
                       std::runtime_error);
 
     // wrong scalar type:
@@ -554,8 +536,9 @@ namespace muSpectre {
     col_hard.register_int_state_field(wrong_scalar_name, nb_steps_to_save, Dim,
                                       QuadPtTag);
 
-    BOOST_CHECK_THROW(this->globalise_real_old_field(wrong_scalar_name, 1),
-                      std::runtime_error);
+    BOOST_CHECK_THROW(
+        this->cell->globalise_real_old_field(wrong_scalar_name, 1),
+        std::runtime_error);
   }
 
   BOOST_AUTO_TEST_SUITE_END();
