@@ -59,8 +59,107 @@ namespace muSpectre {
 
   /* ---------------------------------------------------------------------- */
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(constructor_test, fix, fixlist, fix) {
-    BOOST_CHECK_NO_THROW(
-        fix::projector.initialise());
+    BOOST_CHECK_NO_THROW(fix::projector.initialise());
+  }
+
+  /* ---------------------------------------------------------------------- */
+  BOOST_FIXTURE_TEST_CASE_TEMPLATE(full_gradient_preservation_test, fix,
+                                   fixlist, fix) {
+    // create a gradient field with a zero mean gradient and verify
+    // that the projection preserves it
+    constexpr Dim_t dim{fix::sdim}, mdim{fix::mdim};
+    static_assert(
+        dim == fix::mdim,
+        "These tests assume that the material and spatial dimension are "
+        "identical");
+    using Fields = muGrid::GlobalFieldCollection;
+    using RealFieldT = muGrid::RealField;
+    using FourierFieldT = muGrid::TypedFieldBase<Complex>;
+    using FieldMap =
+        muGrid::MatrixFieldMap<Real, Mapping::Mut, mdim, 1, IterUnit::Pixel>;
+    using FieldMapGradFourier =
+        muGrid::MatrixFieldMap<Complex, Mapping::Mut, mdim, mdim,
+                               IterUnit::Pixel>;
+    using FieldMapDispFourier =
+        muGrid::MatrixFieldMap<Complex, Mapping::Const, mdim, 1,
+                               IterUnit::Pixel>;
+    Fields fields{1};
+    RealFieldT & f_disp{fields.register_real_field("displacement", mdim)};
+    RealFieldT & f_grad{fields.register_real_field("gradient", mdim * mdim)};
+    RealFieldT & f_var{
+        fields.register_real_field("working field", mdim * mdim)};
+
+    fields.initialise(fix::projector.get_nb_domain_grid_pts(),
+                      fix::projector.get_nb_subdomain_grid_pts(),
+                      fix::projector.get_subdomain_locations());
+
+    // create a random displacement field with zero mean
+    f_disp.eigen_vec().setRandom();
+
+    FieldMap disp{f_disp};
+    disp -= disp.mean();
+
+    // differentiate displacement field in fourier space
+    auto && fft_engine{fix::projector.get_fft_engine()};
+    FourierFieldT & f_disp_q{
+        fft_engine.fetch_or_register_fourier_space_field("fourier disp", mdim)};
+    FourierFieldT & f_grad_q{fft_engine.fetch_or_register_fourier_space_field(
+        "fourier grad", mdim * mdim)};
+    FieldMapGradFourier grad_q{f_grad_q};
+    FieldMapDispFourier disp_q{f_disp_q};
+    fix::projector.initialise();
+
+    fft_engine.fft(f_disp, f_disp_q);
+
+    BOOST_CHECK_LE(tol, f_disp_q.eigen_vec().squaredNorm());
+
+    auto && nb_domain_grid_pts{fix::projector.get_nb_domain_grid_pts()};
+    muFFT::FFT_freqs<dim> fft_freqs{nb_domain_grid_pts};
+    using Vector_t = typename muFFT::FFT_freqs<dim>::Vector;
+    const Vector_t grid_spacing(
+        eigen((fix::projector.get_domain_lengths() / nb_domain_grid_pts)
+                  .template get<dim>()));
+    auto && gradient_operator{fix::projector.get_gradient()};
+    for (auto && ccoord_disp_grad :
+         akantu::zip(fft_engine.get_fourier_pixels()
+                         .template get_dimensioned_pixels<dim>(),
+                     disp_q, grad_q)) {
+      auto && ccoord{std::get<0>(ccoord_disp_grad)};
+      auto && u{std::get<1>(ccoord_disp_grad)};
+      auto && g{std::get<2>(ccoord_disp_grad)};
+      const Vector_t xi((fft_freqs.get_xi(ccoord).array() /
+                         eigen(nb_domain_grid_pts.template get<dim>())
+                             .array()
+                             .template cast<Real>())
+                            .matrix());
+
+      // compute derivative operator
+      Eigen::Matrix<Complex, dim, 1> diffop;
+      for (Index_t i = 0; i < dim; ++i) {
+        diffop[i] = gradient_operator[i]->fourier(xi) / grid_spacing[i];
+      }
+      g = (diffop * u.transpose()).transpose();
+      // We need to add I to the term, because this field has a net
+      // zero gradient, which leads to a net -I strain
+      g = 0.5 * (g.transpose() + g).eval();
+    }
+
+    fft_engine.ifft(f_grad_q, f_grad);
+
+    f_var = f_grad;
+
+    fix::projector.apply_projection(f_var);
+
+    Real error{
+        muGrid::testGoodies::rel_error(f_var.eigen_vec(), f_grad.eigen_vec())};
+    if (not(error <= tol)) {
+      std::cout << "Projection failure:" << std::endl
+                << "reference gradient = " << std::endl
+                << f_grad.eigen_vec().transpose() << std::endl
+                << "projected gradient = " << std::endl
+                << f_var.eigen_vec().transpose() << std::endl;
+    }
+    BOOST_CHECK_LE(error, tol);
   }
 
   /* ---------------------------------------------------------------------- */
@@ -96,8 +195,8 @@ namespace muSpectre {
         "identical");
     using Fields = muGrid::GlobalFieldCollection;
     using FieldT = muGrid::RealField;
-    using FieldMap = muGrid::MatrixFieldMap<Real, Mapping::Mut, mdim, mdim,
-                                            IterUnit::SubPt>;
+    using FieldMap =
+        muGrid::MatrixFieldMap<Real, Mapping::Mut, mdim, mdim, IterUnit::SubPt>;
     using Vector = Eigen::Matrix<Real, dim, 1>;
 
     Fields fields{sdim};
@@ -110,7 +209,8 @@ namespace muSpectre {
     FieldMap grad(f_grad);
     FieldMap var(f_var);
 
-    fields.initialise(fix::projector.get_nb_subdomain_grid_pts(),
+    fields.initialise(fix::projector.get_nb_domain_grid_pts(),
+                      fix::projector.get_nb_subdomain_grid_pts(),
                       fix::projector.get_subdomain_locations());
 
     muFFT::FFT_freqs<dim> freqs{fix::projector.get_nb_domain_grid_pts(),
@@ -136,8 +236,7 @@ namespace muSpectre {
 
       // We need to add I to the term, because this field has a net
       // zero gradient, which leads to a net -I strain
-      g = 0.5 * ((g - g.Identity()).transpose() + (g - g.Identity())).eval() +
-          g.Identity();
+      g = 0.5 * (g.transpose() + g).eval();
       v = g;
     }
 
